from Globals import *
from JobCenter import McServicePositionGauge, PicLoaderScale



class LCDMusicCenterMain(Screen):

	if ARM:
	
		skin = """
			<screen name="LCDMusicCenterMain_ARM" backgroundColor="#00000000" position="0,0" size="400,240">
				<widget name="classname" position="5,5" size="400,26" halign="center" valign="top" font="SansReg;22" foregroundColor="#15FF11" transparent="1"/>
				<eLabel backgroundColor="foreground" position="0,31" size="400,1"/>
				<widget name="text" position="10,100" size="380,40" halign="center" valign="center" font="SansReg;28"/>
			</screen>"""
			
	else:
	
		skin = """
			<screen name="LCDMusicCenterMain" position="0,0" size="132,64">
				<widget name="classname" position="2,0" size="128,14" font="SansReg;12"/>
				<widget name="text" position="2,32" size="128,14" font="SansReg;12"/>
			</screen>"""


	def __init__(self, session, parent):
	
		Screen.__init__(self, session)
		
		self["classname"] = Label(parent.__class__.__name__)
		self["text"] = Label()


	def setText(self, text):
		self["text"].setText(text)



class LCDPlayer(Screen):

	if ARM:

		skin = '''
			<screen name="LCDPlayer_ARM" backgroundColor="#00000000" position="0,0" size="400,240">
				<widget name="classname" position="5,0" size="400,26" halign="center" valign="top" font="SansReg;22" foregroundColor="#15FF11" transparent="1"/>
				<eLabel backgroundColor="foreground" position="0,26" size="400,1"/>
				<widget name="artist" position="5,30" size="1200,32" halign="left" valign="center" font="SansReg;28" foregroundColor="#15FF11" transparent="1"/>
				<widget name="title" position="5,62" size="1200,32" halign="left" valign="center" font="SansReg;28" foregroundColor="#15FF11" transparent="1"/>
				
				<widget name="album" position="130,110" size="400,30" halign="left" valign="center" font="SansReg;22" foregroundColor="#15FF11" transparent="1"/>
				<widget name="genre" position="130,140" size="188,30" halign="left" valign="center" font="SansReg;22" foregroundColor="#15FF11" transparent="1"/>

				<widget name="date" position="320,110" size="400,30" halign="right" valign="center" font="SansReg;22" foregroundColor="#15FF11" transparent="1"/>
				<widget name="bitrate" position="320,140" size="77,30" halign="right" valign="center" font="SansReg;22" foregroundColor="#15FF11" transparent="1"/>
				
				<widget name="cover" position="5,100" size="120,120" transparent="1" zPosition="10"/>
				
				<widget source="session.CurrentService" render="Label" position="130,190" size="200,30" zPosition="24" font="SansReg;20" halign="left" foregroundColor="#f0f0f0" transparent="1" >
					<convert type="ServicePosition">Position,ShowHours</convert>
				</widget>
				<widget source="session.CurrentService" render="Label" position="200,190" size="200,30" zPosition="1" font="SansReg;20" halign="right" foregroundColor="#f0f0f0" transparent="1" >
					<convert type="ServicePosition">Length,ShowHours,HideZero</convert>
				</widget>
				<widget name="PositionGauge" position="130,220" size="270,5" pointer="/usr/lib/enigma2/python/Plugins/Extensions/MusicCenter/images/progressbar.png:270,0" 
					seek_pointer="/usr/lib/enigma2/python/Plugins/Extensions/MusicCenter/images/progressbar.png:270,0" transparent="1"/>
			</screen>'''

	else:	
	
		skin = """
			<screen name="LCDPlayer" position="0,0" size="132,64">
				<widget name="classname" position="2,0" size="128,14" font="SansReg;12"/>
				<widget name="artist" position="2,14" size="128,14" font="SansReg;12"/>
				<widget name="title" position="2,28" size="128,14" font="SansReg;12"/>
				<widget name="album" position="2,42" size="88,12" font="SansReg;10"/>
				<widget name="date" position="90,42" size="42,12" font="SansReg;10"/>
				<widget name="genre" position="2,54" size="88,10" font="SansReg;8"/>
				<widget name="bitrate" position="90,54" size="42,10" font="SansReg;8"/>
			</screen>"""


	def __init__(self, session, parent):
	
		#logger.debug('LCDPlayer]__init__]parent:{}'.format(parent.keys()))
		Screen.__init__(self, session)
		
		self["classname"] = Label(parent.__class__.__name__)
		self["artist"] = Label()
		self["title"] = Label()
		self["album"] = Label()
		self["date"] = Label()
		self["genre"] = Label()
		self["bitrate"] = Label()
		self.parent = parent
		if ARM:
			self['cover'] = PicLoaderScale(transparent = False, name = 'MC_ExtLCDScreen')
			self["PositionGauge"] = McServicePositionGauge(session.nav)
			
			
	def updateSlentry(self, slentry):
		if slentry.filetype in ('MP3','FLAC', 'OGG', 'M4A', 'WMA'):
			logger.debug('LCDPlayer]updateSlentry]Player filetype:{}'.format(slentry.filetype))
			album_year_albumartist, playcount = joinDisplaytextFiles(slentry)
			self["artist"].setText(slentry.artist)
			self["title"].setText(slentry.title)
			self["album"].setText(album_year_albumartist)
			self["date"].setText(slentry.date)
			self["genre"].setText(slentry.genre)
			self["bitrate"].setText(slentry.bitrate)
			
		else:
			logger.debug('LCDPlayer]updateSlentry]SPlayer filetype:{}'.format(slentry.filetype))
			stationname, bitrateandcodec, stats, provider = joinDisplaytextStream(slentry)
			self['artist'].setText(slentry.artist)
			self['title'].setText(slentry.title)
			self['album'].setText(stationname)
			self['genre'].setParsedText(slentry.genre)
			self['bitrate'].setParsedText(bitrateandcodec)
			self["date"].setText(provider)			


	def updateCover(self, coverfn):
		if ARM:
			if coverfn != 'hide':
				self['cover'].doshow(coverfn, forceshow = True)
			else:
				self['cover'].dohide()
