# -*- coding: UTF-8 -*-

#  Music Center E2
#
#  Coded by bobo71 (c) 2014
#  Support: www.dreambox-tools.info
#
#  All Files of this Software are licensed under the Creative Commons
#  Attribution-NonCommercial-ShareAlike 3.0 Unported
#  License if not stated otherwise in a Files Head. To view a copy of this license, visit
#  http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a letter to Creative
#  Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.

#  Additionally, this plugin may only be distributed and executed on hardware which
#  is licensed by Dream Multimedia GmbH.

#  This plugin is NOT free software. It is open source, you are allowed to
#  modify it (if you keep the license), but it may not be commercially
#  distributed other than under the conditions noted above.
#  This applies to the source code as a whole as well as to parts of it, unless
#  explicitely stated otherwise.


from Globals import *
import LCDScreens
from Help import Helps
from JobCenter import *
from JobCenter2 import *
from myLogger import logger
from SelectPath import SelectPath
from Setup import MusicCenterSetup
from SQLConnector import execSQLCommandWithReturn, execSQLCommand, execSQLCommandMany
from tagger import getID3Tags

import Jukebox
import Player
import RadioDe
import RadioBrowser
import SoundCloud
import LautFM
#
#gst-launch-1.0 -v filesrc location="/hoeck/Niclas-2tb/movie/Konzerte/20180412_-_ECHO_2018_-_DER_DEUTSCHE_MUSIKPREIS.ts" ! decodebin ! audioconvert ! lamemp3enc ! filesink location="/hoeck/Local-320gb/Musik/Helene Fischer - On Echo 2018.mp3"
#

# mobile devices
from Screens.Console import Console
from enigma import eFileWatch, eFileEvent

eConsoleAppContainerInstance = eConsoleAppContainer()


class MusicCenterRsyncConsole(Console):
	# Code base from Merlin Console
	
	if RESOLUTIONx>1800:
		skin = """
			<screen position="center,center" size="1300,800" title="Rsync execution..." >
				<widget name="text" position="0,0" size="1300,800" font="Console;21" />
			</screen>"""
	else:
		skin = """
			<screen position="center,center" size="820,560" title="Command execution..." >
				<widget name="text" position="0,0" size="820,560" font="Console;14" />
			</screen>"""

	
	def __init__(self, session, title = "Console", cmdlist = None, finishedCallback = None, closeOnSuccess = False):
	
		Console.__init__(self, session, title = "Console", cmdlist = None, finishedCallback = None, closeOnSuccess = False)
		Screen.__init__(self, session)

		self.finishedCallback = finishedCallback
		self.closeOnSuccess = closeOnSuccess

		self["text"] = ScrollLabel("")
		self["actions"] = ActionMap(["WizardActions", "DirectionActions"], 
		{
			"ok": self.cancel,
			"back": self.cancel,
			"up": self["text"].pageUp,
			"down": self["text"].pageDown
		}, -1)
		
		self.cmdlist = cmdlist
		self.newtitle = title
		
		self.onShown.append(self.updateTitle)
		
		#self.container = eConsoleAppContainerInstance ###################################
		self.run = 0
		self.appClosed_conn = eConsoleAppContainerInstance.appClosed.connect(self.runFinished)
		self.dataAvail_conn = eConsoleAppContainerInstance.dataAvail.connect(self.dataAvail)
		self.dataSent_conn = None
		#self.onLayoutFinish.append(self._onLayoutFinishAppend) # dont start before gui is finished

	def dataAvail(self, text):
		self["text"].setText(self["text"].getText() + text)
		self["text"].lastPage()
		logger.debug('MusicCenterRsyncConsole]dataAvail]{}'.format(text.strip().replace('\n','')))

		if text.lower().endswith('do you want to continue connecting? (y/n) '):
			self.session.openWithCallback(self.processAnswer, MessageBox, _("Host is not in the trusted hosts file. Do you want to continue?"), MessageBox.TYPE_YESNO)

	def processAnswer(self, retval):
		self.dataSent_conn = eConsoleAppContainerInstance.dataSent.connect(self.processInput)
		if retval:
			eConsoleAppContainerInstance.write("y",1)
		else:
			eConsoleAppContainerInstance.write("n",1)
	
	def cancel(self):
		#self.container.kill()
		self.appClosed_conn = None
		self.dataAvail_conn = None
		self.close()
		

class MusicCenterAddToDatabase(Screen):

	if RESOLUTIONx>1800:

		skin="""<screen name="MusicCenterAddToDatabase" position="center,center" size="1600,420" title="Add music files to MusicCenter database">
				<ePixmap pixmap="skin_default/buttons/red.png" position="0,0" zPosition="0" size="210,60" scale="1" transparent="1" alphatest="on" />
				<ePixmap pixmap="skin_default/buttons/green.png" position="210,0" zPosition="0" size="210,60" scale="1" transparent="1" alphatest="on" />
				<widget name="output" position="10,50" size="1500,350" valign="center" halign="center" font="SansReg;27" />
				<widget render="Label" source="key_red" position="0,0" size="210,60" zPosition="5" valign="center" halign="center" backgroundColor="red" font="SansReg;30" transparent="1" foregroundColor="white" />
				<widget render="Label" source="key_green" position="210,0" size="210,60" zPosition="5" valign="center" halign="center" backgroundColor="red" font="SansReg;30" transparent="1" foregroundColor="white" />
			</screen>"""

	else:
		skin="""<screen name="MusicCenterAddToDatabase" position="center,center" size="1067,280" title="Add music files to MusicCenter database">
				<ePixmap pixmap="skin_default/buttons/red.png"    position="0,0" zPosition="0" size="140,40" scale="1" transparent="1" alphatest="on" />
				<ePixmap pixmap="skin_default/buttons/green.png"  position="140,0" size="140,40" zPosition="0" scale="1" transparent="1" alphatest="on" />
				<widget name="output"                             position= "7,33" size="1000,234" valign="center" halign="center" font="SansReg;18" />
				<widget render="Label" source="key_red"           position=  "0,0"  size= "140,40" zPosition="5" valign="center" halign="center" backgroundColor="red" font="SansReg;20" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-1,-1" />
				<widget render="Label" source="key_green"         position="140,0"  size= "140,40" zPosition="5" valign="center" halign="center" backgroundColor="red" font="SansReg;20" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-1,-1" />
			</screen>"""

	def __init__(self, session, initDir=None, autoclose=False, isCleanDB=False):
		logger.info('MusicCenterAddToDatabase]__init__]initDir:{} autoclose:{} isCleanDB:{}'.format(initDir, autoclose, isCleanDB))
		Screen.__init__(self, session)

		self["actions"]=ActionMap(["WizardActions", "ColorActions"],
		{
			"back": self.cancel,
			"green": self.green,
			"red": self.cancel,
			"ok": self.green,
		}, -1)

		self["output"]=Label()
		self.initDir=initDir
		self.autoclose=autoclose
		self.isCleanDB=isCleanDB

		if isCleanDB:
			self.pathToDatabase_mc_conn=pathToDatabase.MessagePump.recv_msg.connect(self.gotThreadMsg_isCleanDB)
			self["key_red"]=StaticText("")
			self["key_green"]=StaticText("")
		else:
			self.pathToDatabase_mc_conn=pathToDatabase.MessagePump.recv_msg.connect(self.gotThreadMsg)
			self["key_red"]=StaticText(_("Cancel"))
			self["key_green"]=StaticText(_("Background"))

		self.onClose.append(self.__onClose)
		self.onLayoutFinish.append(self._onLayoutFinishAppend)

	def _onLayoutFinishAppend(self):
		if self.isCleanDB:
			logger.info('MusicCenterAddToDatabase]_onLayoutFinishAppend]CleanDB')
			pathToDatabase.Start(None)
		elif self.initDir is not None:
			logger.info('MusicCenterAddToDatabase]_onLayoutFinishAppend]Scan directory')
			pathToDatabase.Start(self.initDir)
		else:
			logger.info('MusicCenterAddToDatabase]_onLayoutFinishAppend]Show only Screen')

	def gotThreadMsg(self, msg):# 0->State, 1->Message,
		msg=pathToDatabase.Message.pop()
		#logger.info('MusicCenterAddToDatabase]gotThreadMsg:{}'.format(msg))
		if msg[0] in ('THREAD_WORKING',):
			self["output"].setText(msg[1])

		elif msg[0] in ('THREAD_FINISHED', 'THREAD_STOPANDSAVE', 'THREAD_EXCEPTED'):
			logger.info('MusicCenterAddToDatabase]gotThreadMsg, not good here what is wrong?-->{}'.format(msg))
			self["output"].setText(msg[1])
			self["key_red"].setText(_("Close"))
			self["key_green"].setText(_("Close"))
			if self.autoclose:
				self.close()
		else:
			logger.error('MusicCenterAddToDatabase]gotThreadMsg, not good here what is wrong?-->{}'.format(msg))

	def gotThreadMsg_isCleanDB(self, msg):
		msg=pathToDatabase.Message.pop()
		logger.info('MusicCenterAddToDatabase]gotThreadMsg_isCleanDB]msg->%s' %str(msg))
		if msg[1] is not None:
			self["output"].setText(msg[1])
		if msg[0] in ('THREAD_FINISHED', 'THREAD_EXCEPTED'):
			self["key_green"].setText("Close")
			self["key_red"].setText("Close")
			self.isCleanDB=False

	def green(self):
		if not self.isCleanDB:
			self.close()

	def cancel(self):
		if not self.isCleanDB:
			if pathToDatabase.isRunning:
				pathToDatabase.CancelScanDB()
			self.close()

	def __onClose(self):
		logger.info('MusicCenterAddToDatabase]__onClose]...')
		self.pathToDatabase_mc_conn=None


class AppList(MenuList):


	def __init__(self, list = [], enableWrapAround = False):

		logger.info('AppList]__init__]')
		MenuList.__init__(self, list, enableWrapAround, eListboxPythonMultiContent)

		self.l.setItemHeight(150)
		self.l.setFont(0, gFont("SansReg", 39))
		self.l.setBuildFunc(self.buildEntry)
		self.onSelectionChanged = [ ]
		self.list = list


	def buildEntry(self, entry):
	
		res = [ entry ]
		selcolor = parseColor("#ffffff").argb()# white
		#color = parseColor("#C6C6C6").argb()#fast weiß
		color = parseColor("#636363").argb()#grau
		ptrbmp=LoadPixmap(cached=False, path=entry[1])
		res.append((eListboxPythonMultiContent.TYPE_PIXMAP,	0,  25, 100, 100, ptrbmp))
		#res.append((eListboxPythonMultiContent.TYPE_TEXT, 0,0, 100, 100, 0, RT_HALIGN_CENTER|RT_VALIGN_CENTER, entry[2], color, selcolor)) # artist + title

		return res
'''
class AlphabetList(AppList):

	def __init__(self, list = [], enableWrapAround = False):
	
		AppList.__init__(self, list, enableWrapAround)

	def buildEntry(self, entry):
	
		res = [ None ]
		selcolor = parseColor("#ffffff").argb()# white
		#color = parseColor("#C6C6C6").argb()#fast weiß
		color = parseColor("#636363").argb()#grau
		res.append((eListboxPythonMultiContent.TYPE_TEXT,       0,0, 100,150, 0, RT_HALIGN_CENTER|RT_VALIGN_CENTER, entry[0], color, selcolor)) # artist + title

		return res
'''
		
SKINPATH='/usr/lib/enigma2/python/Plugins/Extensions/MusicCenter/pics/'

class MusicCenterMain(Screen):

	if RESOLUTIONx>1800:################################################################

		skin="""
			<screen name="MusicCenterMain" position="0,0" size="1920,1080" flags="wfNoBorder" backgroundColor="#ff000000">
				<widget name="logo" position="0,-5" size="450,50" font="SansReg;33" foregroundColor="#FFFFFF" backgroundColor="#00000000" halign="center" valign="center" zPosition="3" cornerRadius="15"/>
				<widget name="version" position="1470,-5" size="450,50" font="SansReg;33" foregroundColor="#FFFFFF" backgroundColor="#00000000" halign="center" valign="center" zPosition="3" cornerRadius="15"/>
				<widget name="applist" position="0,483" size="1920,126" transparent="0" backgroundColor="#20000000" zPosition="3" coverSize="500,100" selectedCoverScale="1.2" unselectedCoverDimm="0.8" coverBeginPosition="300,63" coverflowCurrentPosition="960,63" coverflowCurrentXDistance="80" coverflowCurrentCenterDistance="80" style="1" noCoverAvailablePic="/usr/lib/enigma2/python/Plugins/Extensions/MusicCenter/images/No Poster.png" />
			</screen>"""
			
	else:
		skin="""
			<screen name="MusicCenterMain" position="0,0" size="1280,720" flags="wfNoBorder" backgroundColor="#ff000000">
				<widget name="logo" position="0,-3" size="300,33" font="SansReg;22" foregroundColor="#FFFFFF" backgroundColor="#00000000" halign="center" valign="center" zPosition="3" cornerRadius="10"/>
				<widget name="version" position="980,-3" size="300,33" font="SansReg;22" foregroundColor="#FFFFFF" backgroundColor="#00000000" halign="center" valign="center" zPosition="3" cornerRadius="10"/>
				<widget name="applist" position="0,322" size="1280,84" transparent="0" backgroundColor="#20000000" zPosition="3" coverSize="334,67" selectedCoverScale="1.2" unselectedCoverDimm="0.8" coverBeginPosition="200,42" coverflowCurrentPosition="640,42" coverflowCurrentXDistance="56" coverflowCurrentCenterDistance="56" style="1" noCoverAvailablePic="/usr/lib/enigma2/python/Plugins/Extensions/MusicCenter/images/No Poster.png" />
			</screen>"""


	def __init__(self, session, servicelist):

		logger.info('MusicCenterMain]init]...')
		self.session=session

		Screen.__init__(self, session)

		self["applist"] = CoverCollection()
		#self["alphabet"] = AlphabetList()

		self["actions"]=ActionMap(["WizardActions", "DirectionActions", "ColorActions", "EPGSelectActions", "InfobarAudioSelectionActions", "HelpActions"],
		{
			"ok": self.okPressed,
			"back": self.closing,
			"input_date_time": self.menuPressed,
			"audioSelection": self.audioPressed,
			"left": self.left,
			"right": self.right,
			"up": self.up,
			"down": self.down,
			"displayHelp": self.audioPressed,
		}, -1)

		self.onLayoutFinish.append(self._onLayoutFinishAppend)
		self.onClose.append(self.__onClose)

		self.currentservice = self.session.nav.getCurrentlyPlayingServiceReference()
		
		self.servicelist = servicelist
		self.TOKEN=None

		#self["video"] = VideoWindow(fb_width = getDesktop(0).size().width(), fb_height = getDesktop(0).size().height())

		self.player=None
		self.lastplaylist=None
		self['logo']=Label('MusicCenter')
		self['version']=Label('{}'.format(MC_VERSION))
		self.session.nav.SleepTimer.on_state_change.append(self.sleepTimerEntryOnStateChange)

	def _onLayoutFinishAppend(self):
		logger.info('MusicCenterMain]_onLayoutFinishAppend]...')
		d=SimpleThread(lambda:self.initCacheAndDirectorys()).start()
		#self.instance.setShowHideAnimation("slide_top_to_bottom")
		self['logo'].instance.setShowHideAnimation("slide_top_to_bottom")
		self['version'].instance.setShowHideAnimation("slide_top_to_bottom")
		self['applist'].instance.setShowHideAnimation("slide_zoom_left_to_right")
		
		self["applist"].connectSelChanged(self.updatePosterAndLCD)
		ref = eServiceReference(0x1013, 0, SKINPATH+'music')
		self.session.nav.playService(ref)
		self.buildAppList()

	def initCacheAndDirectorys(self):
	
		if pathExists('/tmp/mc/'):
			logger.info('MusicCenterMain]initCacheAndDirectorys]clear old cache /tmp/mc/')
			sh_rmtree('/tmp/mc/')
			
		if not pathExists('/tmp/mc/'):
			logger.info('MusicCenterMain]initCacheAndDirectorys]create new tmp cache on /tmp/mc/')
			if not createDir('/tmp/mc/'):
				raise ValueError('MusicCenterMain]initCacheAndDirectorys] createDir error on /tmp/mc/')
				
		logger.info('MusicCenterMain]initCacheAndDirectorys]validate nessesary dirs')
		for basepath, dirs in ((config.plugins.musiccenter.downloadcache.value, ('artistpics', 'artistpics/Default', 'cache', 'covers', 'favoritestationicons', 
			'screensaver', 'shazam', 'waves')), (config.plugins.musiccenter.databasepath.value, ('',)),(config.plugins.musiccenter.defaultfilebrowserpath.value, ('',)),
			(config.plugins.musiccenter.streamripper_base_directory.value, ('',))):
			for dir in dirs:
				path=os_path_join(basepath, dir)		
				if not pathExists(path):
					logger.info('MusicCenterMain]initCacheAndDirectorys] createDir {}'.format(path))
					if not createDir(path):
						raise ValueError('MusicCenterMain]initCacheAndDirectorys] createDir error on {}'.format(path))
		logger.info('MusicCenterMain]initCacheAndDirectorys]done...')

	def checkDatabaseScannerIsRunning(self, mode):
		if mode in (self.openFilebrowser, self.openJukebox) and  pathToDatabase.isRunning:
			self.session.openWithCallback(self.callbackScanner, MessageBox, _("Your music collection will be scanned.\nShow Scanner Screen?"), MessageBox.TYPE_YESNO)
			return True
		else:
			return False
			
	def callbackScanner(self, ret):
		if ret:
			self.session.open(MusicCenterAddToDatabase)

	def left(self):
	      self["applist"].MoveLeft()

	def right(self):
	      self["applist"].MoveRight()

	def down(self):
	      self["applist"].MoveDown()

	def up(self):
	      self["applist"].MoveUp()

	def buildAppList(self):
	
		list=[]	
		posterlist=[]
		posterlist.append(SKINPATH+'Filebrowser.png')
		list.append(((self.openFilebrowser, 'Filebrowser'),))
		posterlist.append(SKINPATH+'Jukebox.png')
		list.append(((self.openJukebox, 'Jukebox'),))
		posterlist.append(SKINPATH+'Radiobrowser.png')
		list.append(((self.openRadiobrowser, 'Radiobrowser'),))
		posterlist.append(SKINPATH+'Radio.de.png')
		list.append(((self.openRadioDe, 'Radio.de'),))
		posterlist.append(SKINPATH+'Laut.fm.png')
		list.append(((self.openLautFM, 'Laut.fm'),))	
		
		self["applist"].setList(list, posterlist, 0)
		
		self.updatePosterAndLCD(0)


	def okPressed(self):
		sel = self["applist"].getCurrent()
		if sel is None:
			return
		self.checkDatabaseScannerIsRunning(sel[0])
		logger.info('''MusicCenterMain]okPressed]exec"self.{}()"'''.format(sel[0]))
		sel[0]()
 
 
	def browseAlbums(self):
		pass
	
	def openFilebrowser(self):
		self.session.openWithCallback(self.callbacks, MusicCenterFileList, self.player, self.currentservice, self.servicelist)

	def opendbModewithstartplaylist(self):
		if JCcheckModuleIsUpdated('DatabaseBrowser'):
			reload(DatabaseBrowser)
		self.session.openWithCallback(self.callbacks, DatabaseBrowser.DatabaseBrowser, self.player, True, self.currentservice, self.servicelist)

	def openJukebox(self):
		if JCcheckModuleIsUpdated('Jukebox'):
			reload(Jukebox)
		self.session.openWithCallback(self.callbacks,Jukebox.Jukebox, self.player, self.currentservice, self.servicelist)

	def openLautFM(self):
		if JCcheckModuleIsUpdated('LautFM'):
			reload(LautFM)
		self.session.openWithCallback(self.callbacks, LautFM.LautFM, self.player, self.currentservice, self.servicelist)


	def openSoundCloud(self):
		if JCcheckModuleIsUpdated('SoundCloud'):
			reload(LautFM)
		self.session.openWithCallback(self.callbacks, SoundCloud.SoundCloud, self.player, self.currentservice, self.servicelist, self.TOKEN)

		
	def openRadiobrowser(self):
		if JCcheckModuleIsUpdated('RadioBrowser'):
			reload(RadioBrowser)
		self.session.openWithCallback(self.callbacks, RadioBrowser.RadiobrowserMain, self.player, self.currentservice, self.servicelist)

	def openRadioDe(self):
		if JCcheckModuleIsUpdated('RadioDe'):
			reload(RadioDe)
		self.session.openWithCallback(self.callbacks, RadioDe.RadioDeMain, self.player, self.currentservice, self.servicelist)

	def callbacks(self, player, TOKEN=None):
		logger.info('MusicCenterMain]callbacks...')
		if TOKEN is not None:
			self.TOKEN=TOKEN
		self.player=player


	def sleepTimerEntryOnStateChange(self, timer):
		if timer.state == TimerEntry.StateEnded:
			self.close()

	def getCurrentSelection(self):
		try: sel=self['list'].l.getCurrentSelection()[0]
		except: sel=None
		return sel

	def menuPressed(self):
		options=[(_("Configuration"), self.openConfig),]
		self.session.openWithCallback(self.menuCallback, ChoiceBox,list=options)

	def menuCallback(self, ret):
		ret and ret[1]()

	def audioPressed(self):
		if self.player is not None and self.player.songlist:
			self.session.execDialog(self.player)
# end
	def showScannerCallback(self):
		self.session.openWithCallback(self.scannerClosed, MusicCenterAddToDatabase, None)

	def openConfig(self):
		self.session.openWithCallback(self.setupFinished, MusicCenterSetup)

	def setupFinished(self, result):
		if result:
			self._onLayoutFinishAppend()

	def cleanCovercache(self, cachedir, maxcachesizeMB=500):
		# 500MB default cache size
		logger.debug('MusicCenterMain]cleanCovercache]start, check size...')
		cachefilelist=[(int(os_stat(cachedir+f).st_size),os_stat(cachedir+f).st_ctime, cachedir+f) for f in os_listdir(cachedir)] # list files(size,ctime,filename
		cachesizeMB=sum([f[0] for f in cachefilelist])/1024/1024.0
		if cachesizeMB > maxcachesizeMB:
			logger.info('MusicCenterMain]cleanCovercache]cachesize is ->%dMB, start cleanup!' %cachesizeMB)
			cachefilelist.sort(key=lambda a:a[1]) # ctime ascend
			while cachesizeMB>maxcachesizeMB:
				cachefile=cachefilelist.pop(0)
				logger.debug('MusicCenterMain]cleanCovercache]remove -> %s' %cachefile[2])
				os_remove(cachefile[2])
				cachesizeMB-=cachefile[0]/1024/1024.0
			logger.debug('MusicCenterMain]cleanCovercache]stop removing new size is -> %d' %cachesizeMB)
		else:
			logger.info('MusicCenterMain]cleanCovercache]cachesize is ->%dMB, nothing to do.' %cachesizeMB)
		logger.debug('MusicCenterMain]cleanCovercache]done')

	def closePlayerNow(self):
		if self.player.seekstate!=self.player.SEEK_STATE_STOP:# -2 = no playlist
			logger.info('MusicCenterMain]closePlayerNow]seekstate {} call updateRatingPlaycounterWaveCover and stopPlay'.format(self.player.seekstate))
			self.player.stopPlay()
			
		logger.info('MusicCenterMain]closePlayerNow]')
		self.session.deleteDialog(self.player)
		self.player=None

	def closing(self):
		logger.info('MusicCenterMain]closing]...')
		self.close()

	def __onClose(self):
		logger.info('MusicCenterMain]__onClose]Start')
		self.session.nav.SleepTimer.on_state_change.remove(self.sleepTimerEntryOnStateChange)
		if self.player is not None:
			logger.info('MusicCenterMain]__onClose]Remove self.playerinstance...')
			self.closePlayerNow()
		if self.servicelist is None:
			self.session.nav.playService(self.currentservice)
		else:
			current = ServiceReference(self.servicelist.getCurrentSelection())
			self.session.nav.playService(current.ref)

		logger.info('MusicCenterMain]__onClose]clear /tmp/mc/....')
		sh_rmtree('/tmp/mc/')

	def updatePosterAndLCD(self, index):
		sel=self['applist'].getCurrent()
		if sel is not None:
			self.summaries.setText(sel[1])
			#ref = eServiceReference(0x1013, 0, SKINPATH+'music.m1v')
			#self.playService(ref)
			#self.session.nav.playService(ref)
			#ffmpeg -i "/tmp/music_1920.jpg" -r 25 -b 20000 -s hd1080 /usr/lib/enigma2/python/Plugins/Extensions/MusicCenter/pics/music.m1v
			
	def createSummary(self):
		if JCcheckModuleIsUpdated('LCDScreens'):
			reload(LCDScreens)
		return LCDScreens.LCDMusicCenterMain

		
class MC_MultiFileSelectList(BaseMultiFileSelectList):

	def __init__(self, preselectedFiles, directory, showMountpoints = False, matchingPattern = None, showDirectories = True, showFiles = True,  useServiceRef = False, inhibitDirs = False, inhibitMounts = False, isTop = False, enableWrapAround = False, additionalExtensions = None):

		logger.info('MC_MultiFileSelectList]__init__]...')

		BaseMultiFileSelectList.__init__(self, preselectedFiles, directory, showMountpoints, matchingPattern, showDirectories, showFiles,  useServiceRef, inhibitDirs, inhibitMounts, isTop, enableWrapAround, additionalExtensions)

	def inParentDirs(self, dir, parents):
		# override inhibitDirs from absolute to in path...
		dir = os_path_realpath(dir)
		dir=dir.rsplit('/',1)[-1]
		#logger.debug('MC_MultiFileSelectList]inParentDirs:{} parents:{}'.format(dir, parents))
		for p in parents:
			if dir.startswith(p):
				return True
		return False

	def byNameFunc(self, a, b):
		return cmp(b[0][1], a[0][1]) or cmp(a[1][7], b[1][7])

	def sortName(self):
		self.list.sort(self.byNameFunc)
		#self.l.invalidate()
		self.l.setList(self.list)
		self.moveToIndex(0)

	def byDateFunc(self, a, b):
		try:
			stat1 = os_stat(self.current_directory + a[0][0])
			stat2 = os_stat(self.current_directory + b[0][0])
		except:
			return 0
		return cmp(b[0][1], a[0][1]) or cmp(stat2.st_ctime, stat1.st_ctime)

	def sortDate(self):
		self.list.sort(self.byDateFunc)
		#self.l.invalidate()
		self.l.setList(self.list)
		self.moveToIndex(0)


class MusicCenterFileList(Screen, PlaylistBase):

	if RESOLUTIONx >1800:

		skin='''
			<screen name="MusicCenterFileList1" position="0,0" size="1920,1080" flags="wfNoBorder" backgroundColor="#ff000000" title="MC_Filelist">
				<!-- head  -->
					<widget name="location" position="25,25" size="1870,50" font="SansReg;35" halign="left" valign="center"  foregroundColor="#ffffff" backgroundColor="#40000000" zPosition="2"  cornerRadius="15"/>
				<!-- description  -->
					<eLabel position="1330,90" size="560,480" zPosition="1" foregroundColor="transparent" backgroundColor="#40000000" cornerRadius="15" />
					<widget name="artist" position="1340,100" size="540,80" font="SansReg;34" halign="center" valign="center"  foregroundColor="#ffffff" transparent="1" zPosition="3"  cornerRadius="15"/>
					<widget name="title" position="1340,220" size="540,80" font="SansReg;35" halign="center" valign="center"  foregroundColor="#ffffff" transparent="1" zPosition="3"  cornerRadius="15"/>
					<widget name="genre" position="1340,340" size="540,40" font="SansReg;28" halign="center" valign="center"  foregroundColor="#ffffff" transparent="1" zPosition="3" cornerRadius="15"/>
					<widget name="length" position="1340,410" size="200,40" font="SansReg;28" halign="center" valign="center"  foregroundColor="#ffffff" transparent="1" zPosition="3" cornerRadius="15"/>
					<widget name="bitrate" position="1540,410" size="150,40" font="SansReg;28" halign="center" valign="center"  foregroundColor="#ffffff" transparent="1" zPosition="3" cornerRadius="15"/>
					<widget name="plays" position="1720,410" size="150,40" font="SansReg;28" halign="center" valign="center"  foregroundColor="#ffffff" transparent="1" zPosition="3" cornerRadius="15"/>
					<widget name="rating" position="1340,500" size="540,40" alphatest="on" zPosition="3" scale="0" pixmaps="~/rating0.png,~/rating1.png,~/rating2.png,~/rating3.png,~/rating4.png,~/rating5.png,~/rating6.png,~/rating7.png,~/rating8.png,~/rating9.png,~/rating10.png,~/rating11.png"/>
					<widget name="coverArt" position="1465,620" size="350,350" zPosition="3"/>
				<!-- Buttons -->
					<widget render="Label" font="SansReg;28" position="60,1000" size="450,32" source="key_red" foregroundColor="#ff0000" transparent="1" halign="center" valign="center" zPosition="3"/>
					<widget render="Label" font="SansReg;28" position="510,1000" size="450,32" source="key_green" foregroundColor="#00ff21" transparent="1" halign="center" valign="center" zPosition="3"/>
					<widget render="Label" font="SansReg;28" position="960,1000" size="450,32" source="key_yellow" foregroundColor="#FFD800" transparent="1" halign="center" valign="center" zPosition="3"/>
					<widget render="Label" font="SansReg;28" position="1410,1000" size="450,32" source="key_blue" foregroundColor="#3D8DFF" transparent="1" halign="center" valign="center" zPosition="3"/>
				<!-- list -->
					<ePixmap position="center,910" size="100%,15%" gradient="#20808080,#20000000,vertical" zPosition="2"/>
					<widget name="filelist" position="25,90" zPosition="3" size="1285,880" scrollbarMode="showOnDemand" backgroundColor="#40000000"/>
					<widget name="waittext" position="center,center" size="300,60" font="SansReg;30" foregroundColor="#ffffff" backgroundColor="#40000000" halign="center" valign="center" zPosition="8" cornerRadius="15"/>
			</screen>'''
	else:
		skin='''
			<screen name="MusicCenterFileList1" position="0,0" size="1280,720" flags="wfNoBorder" backgroundColor="#ff000000" title="MC_Filelist">
				<!-- head  -->
					<widget name="location" position="16,16" size="1247,33" font="SansReg;24" halign="left" valign="center"  foregroundColor="#ffffff" backgroundColor="#40000000" zPosition="2"  cornerRadius="10"/>
				<!-- description  -->
					<eLabel position="887,60" size="373,320" zPosition="1" foregroundColor="transparent" backgroundColor="#40000000" cornerRadius="10" />
					<widget name="artist" position="893,67" size="360,53" font="SansReg;22" halign="center" valign="center"  foregroundColor="#ffffff" transparent="1" zPosition="3"  cornerRadius="10"/>
					<widget name="title" position="893,146" size="360,53" font="SansReg;24" halign="center" valign="center"  foregroundColor="#ffffff" transparent="1" zPosition="3"  cornerRadius="10"/>
					<widget name="genre" position="893,226" size="360,26" font="SansReg;18" halign="center" valign="center"  foregroundColor="#ffffff" transparent="1" zPosition="3" cornerRadius="10"/>
					<widget name="length" position="893,273" size="133,26" font="SansReg;18" halign="center" valign="center"  foregroundColor="#ffffff" transparent="1" zPosition="3" cornerRadius="10"/>
					<widget name="bitrate" position="1026,273" size="100,26" font="SansReg;18" halign="center" valign="center"  foregroundColor="#ffffff" transparent="1" zPosition="3" cornerRadius="10"/>
					<widget name="plays" position="1147,273" size="100,26" font="SansReg;18" halign="center" valign="center"  foregroundColor="#ffffff" transparent="1" zPosition="3" cornerRadius="10"/>
					<widget name="rating" position="893,333" size="360,26" alphatest="on" zPosition="3" scale="0" pixmaps="~/rating0.png,~/rating1.png,~/rating2.png,~/rating3.png,~/rating4.png,~/rating5.png,~/rating6.png,~/rating7.png,~/rating8.png,~/rating9.png,~/rating10.png,~/rating11.png"/>
					<widget name="coverArt" position="977,413" size="233,233" zPosition="3"/>
				<!-- Buttons -->
					<widget render="Label" font="SansReg;18" position="40,667" size="300,22" source="key_red" foregroundColor="#ff0000" transparent="1" halign="center" valign="center" zPosition="3"/>
					<widget render="Label" font="SansReg;18" position="340,667" size="300,22" source="key_green" foregroundColor="#00ff21" transparent="1" halign="center" valign="center" zPosition="3"/>
					<widget render="Label" font="SansReg;18" position="640,667" size="300,22" source="key_yellow" foregroundColor="#FFD800" transparent="1" halign="center" valign="center" zPosition="3"/>
					<widget render="Label" font="SansReg;18" position="940,667" size="300,22" source="key_blue" foregroundColor="#3D8DFF" transparent="1" halign="center" valign="center" zPosition="3"/>
				<!-- list -->
					<ePixmap position="center,606" size="100%,15%" gradient="#20808080,#20000000,vertical" zPosition="2"/>
					<widget name="filelist" position="16,60" zPosition="3" size="857,587" scrollbarMode="showOnDemand" backgroundColor="#40000000"/>
					<widget name="waittext" position="center,center" size="200,40" font="SansReg;20" foregroundColor="#ffffff" backgroundColor="#40000000" halign="center" valign="center" zPosition="8" cornerRadius="10"/>
			</screen>'''

	def __init__(self, session, player, currentservice, servicelist):

		self.session=session
		self.player=player
		self.currentservice=currentservice
		self.servicelist=servicelist
		
		self.skin_path='/usr/lib/enigma2/python/Plugins/Extensions/MusicCenter/images'
		Screen.__init__(self, session)
		PlaylistBase.__init__(self)

		self.selectedDir="/tmp/"
		self.selectedFiles= []

		self.FileEraser=eBackgroundFileEraser.getInstance()
		self.coverLoader=CoverLoader()

		if config.plugins.musiccenter.rememberlastfilebrowserpath.value:
			startupPath=config.plugins.musiccenter.lastusedpath.value
		else:
			startupPath=config.plugins.musiccenter.defaultfilebrowserpath.value
		if not pathExists(startupPath):
			startupPath='/media/'

		self["location"]=Label()
	
		self["artist"]=Label()
		self["title"]=Label()
		self["genre"]=Label()
		self["plays"]=Label()
		self["bitrate"]=Label()
		self["length"]=Label()
		self["rating"]=MultiPixmap()

		self["key_red"]=StaticText("Loesche")
		self["key_green"]=StaticText("Umbenennen")
		self["key_yellow"]=StaticText("Verschiebe/Kopiere")
		self["key_blue"]=StaticText("Lesezeichen")
		
		self.updateLocationInfotimer=eTimer()
		self.musicinfoupdatetimer_mc_conn=self.updateLocationInfotimer.timeout.connect(self.onUpdateLocationtimerTimeout)
		self["coverArt"]=PicLoaderScale(transparent=False, name='Filelistcover')
		self['waittext']=Label()
		self['waittext'].hide()

		matchingPattern='(?i)^.*\.({})$'.format('|'.join(PLAYABLEFORMATS))
		logger.info('Filelist]__init__]init Filelistpath:{} with matchingPattern:{}'.format(startupPath, matchingPattern))
		self["filelist"]= MC_MultiFileSelectList(self.selectedFiles, startupPath, matchingPattern=matchingPattern, inhibitDirs=['.'], enableWrapAround = True)

		self["actions"]=ActionMap(["WizardActions", "DirectionActions", "ColorActions", "MenuActions", "EPGSelectActions", "InfobarActions", "InfobarAudioSelectionActions", "GlobalActions","NumberActions","InfobarChannelSelection","HelpActions"],
		{
			"ok": self.okPressed,
			"back": self.explExit,
			"green": self.ExecRename,
			"red": self.execDelete,
			"blue": self.listBookmarks,
			"yellow": self.go2CPmaniger,
			"menu": self.menuPressed,
			"audioSelection": self.audioPressed,
			"8": self.changeSelectionState,
			"0": self.directoryUp,
			"nextBouquet": self.sortName,
			"prevBouquet": self.sortDate,
			"info" : self.infoPressed,
			"showMovies": self.videoPressed,
			"displayHelp": self.audioPressed,
		}, -1)
			
		self.songlisttmp=[]
		self.foundIndex=""
		self.currfolder=""
		self.hlp={}
		self.playable_songs_count=0
		self.scanPath_mc_conn=scanPath.MessagePump.recv_msg.connect(self.gotThreadMsg)

		self.onClose.append(self.__onClose)
		self.onLayoutFinish.append(self._onLayoutFinishAppend)

	def destroy(self):
		self["coverArt"].destroy()

	def changeSelectionState(self):
		cmd='''mp3gain -c -p -r -d 2.0'''
		pass # self["filelist"].changeSelectionState()
	
	def _onLayoutFinishAppend(self):
		self.instance.setShowHideAnimation("slide_top_to_bottom")
		self["filelist"].onSelectionChanged.append(self.updateLocationInfo)
		self.onUpdateLocationtimerTimeout()

	def updateLocationInfo(self):
		if self.updateLocationInfotimer.isActive():
			self.updateLocationInfotimer.stop()
		self.updateLocationInfotimer.start(220, 1)
		
	def onUpdateLocationtimerTimeout(self):
		currfolder=self["filelist"].getCurrentDirectory()
		if currfolder!=None:
			file=self["filelist"].getFilename()
			canDescent = self["filelist"].canDescent()
			logger.debug('MusicCenterFileList]onUpdateLocationtimerTimeout]canDescent:{}'.format(canDescent))
			if not canDescent: # file
				if file is not None:
					fn=''.join((currfolder, file))
					if not os_path_isdir(fn):
						t0=SimpleThread(lambda:self.getTitleInfoOnDefered(fn))
						t0.deferred.addCallback(self.getTitleInfoOnDeferedAppend,)
						t0.deferred.addErrback(self.errorGetInfoOnDefered, fn, 'getTitleInfoOnDefered')
						t0.start()			
					else:
						sel=self["filelist"].getSelection()
						t0=SimpleThread(lambda:self.getDirectoryinfoOnDefered(sel, currfolder))
						t0.deferred.addCallback(self.getDirectoryInfoOnDeferedAppend,)
						t0.deferred.addErrback(self.errorGetInfoOnDefered, sel[0], 'getDirectoryinfoOnDefered')
						t0.start()
						'''
						t0=SimpleThread(lambda:self.fileNotFoundOnDefered(fn))
						t0.deferred.addCallback(self.getDirectoryInfoOnDeferedAppend,)
						t0.deferred.addErrback(self.errorGetInfoOnDefered, fn, 'fileNotFoundOnDefered')
						t0.start()
						'''
			else: # directory
				sel=self["filelist"].getSelection()
				t0=SimpleThread(lambda:self.getDirectoryinfoOnDefered(sel, currfolder))
				t0.deferred.addCallback(self.getDirectoryInfoOnDeferedAppend,)
				t0.deferred.addErrback(self.errorGetInfoOnDefered, sel[0], 'getDirectoryinfoOnDefered')
				t0.start()
		else:
			self.getDirectoryInfoOnDeferedAppend(('n/a', 'n/a', 'n/a', 'hide'))

	def fileNotFoundOnDefered(self, fn):
		return ('File not found', 'hide')

	def getDirectoryinfoOnDefered(self, sel, currfolder):
		if currfolder.startswith(config.plugins.musiccenter.defaultfilebrowserpath.value):
			currfolder='MyMusic/' + currfolder[len(config.plugins.musiccenter.defaultfilebrowserpath.value):]
			
		if sel[3] in sel[0]:
			dirs=0
			songs=0
			for entry in os_listdir(sel[0]):
				if os_path_isdir(os_path_join(sel[0], entry)):
					dirs+=1	
				elif entry.lower().endswith(PLAYABLEFORMATS):
					songs+=1
	
			return (currfolder, '{} directory(s)'.format(dirs), '{} song(s)'.format(songs), os_path_join(self.skin_path, 'folder_music_x200.png'))
		else:
			return (currfolder, '', '', 'hide')
					
	def getTitleInfoOnDefered(self, filename):
		song_id = -1
		coverfn = self.coverLoader.load(filename)
		if not fileExists(coverfn):
			coverfn = ''.join((self.skin_path, '/no-cover.png'))

		if filename.startswith(config.plugins.musiccenter.defaultfilebrowserpath.value):
			filenameindb = filename[len(config.plugins.musiccenter.defaultfilebrowserpath.value):]
			sql = '''SELECT filename, artist, title, album, albumartist, genre, track, length, bitrate, date, playcount, rating, song_id FROM Songs WHERE filename=? LIMIT 1;'''
			#SELECT filename, artist, title, album, albumartist, genre, track, length, bitrate, date, playcount, rating FROM Songs WHERE filename="2018-1/0526 - Narada Michael Walden - I Shoulda Loved Ya.mp3" LIMIT 1;
			args=(filenameindb,)
			res = execSQLCommandWithReturn(sql, args).fetchone()
			if res: #is in db
				filename, artist, title, album, albumartist, genre, track, length, bitrate, date, playcount, rating, song_id = res
			else:								
				logger.info('Filelist]getTitleInfoOnDefered]no in db found, get id3 now from file->%s' %filename)
				mypath, myfilename=os_path_split(filename)
				ID3=getID3Tags(mypath, myfilename)
				artist=ID3.artist
				title=ID3.title
				album=ID3.album
				albumartist=ID3.albumartist
				genre=ID3.genre
				track=ID3.track
				length=ID3.length
				bitrate=ID3.bitrate
				date=ID3.date
				playcount=ID3.playcount
				rating=calcRating(ID3.rating)
				
		else:
			#logger.info('Filelist]getTitleInfoOnDefered]musicfile not in Musiclib, get id3 now from file->%s' %filename)
			mypath, myfilename=os_path_split(filename)
			ID3=getID3Tags(mypath, myfilename)
			artist=ID3.artist
			title=ID3.title
			album=ID3.album
			albumartist=ID3.albumartist
			genre=ID3.genre
			track=ID3.track
			length=ID3.length
			bitrate=ID3.bitrate
			date=ID3.date
			playcount=ID3.playcount
			rating=calcRating(ID3.rating)
		if album in ('n/a',''):
			#logger.info('Filelist]getTitleInfoOnDefered]set directory name as album')
			album=filename.rsplit('/',2)[1]
			
		if filename.startswith(config.plugins.musiccenter.defaultfilebrowserpath.value):
			filename='MyMusic/' + filename[len(config.plugins.musiccenter.defaultfilebrowserpath.value):]
		
		return (filename, artist, title, album, albumartist, genre, track, length, bitrate, date, playcount, rating, coverfn, song_id)


	def getTitleInfoOnDeferedAppend(self, result):
		filename, artist, title, album, albumartist, genre, track, length, bitrate, date, playcount, rating, coverfn, song_id = result
		self.hlp['song_id_length'] = (song_id, length)
		logger.debug('MusicCenterFileList]getTitleInfoOnDeferedAppend]filename:{}'.format(filename))
		self['artist'].setText(artist)
		self['title'].setText(title)
		self['genre'].setText(genre)
		self["plays"].setText('Plays {}'.format(playcount))	
		self["bitrate"].setText(convertBitrate(bitrate))
		self["length"].setText(convertLength(length))
		self["rating"].setPixmapNum(calcRating(rating))
		self["coverArt"].doshow(coverfn, forceshow=True)
		slentry = Item(artist = artist, title = title, album = album, genre = genre)
		self.updateLCD(slentry, coverfn)


	def getDirectoryInfoOnDeferedAppend(self, result):
		currfolder, dircount, filecount, coverfn = result
		logger.debug('MusicCenterFileList]getDirectoryInfoOnDeferedAppend]currfolder:{} coverfn:{}'.format(currfolder, coverfn))
		self['location'].setText(currfolder)
		self['artist'].setText(dircount)
		self['title'].setText(filecount)	
		self['genre'].setText('')
		self["plays"].setText('')		
		self["bitrate"].setText('')
		self["length"].setText('')
		self["rating"].setPixmapNum(11)
		if coverfn=='hide':
			self["coverArt"].dohide()
		else:
			self["coverArt"].doshow(coverfn, forceshow=True)
		self.updateLCD(Item(), coverfn)

		
	def updateLCD(self, slentry, coverfn):
		self.summaries.updateSlentry(slentry)
		self.summaries.updateCover(coverfn)


	def errorGetInfoOnDefered(self, error, filename, function):
		self.hlp['song_id_length'] = None		
		logger.error('Filelist]errorGetInfoOnDefered]function:{} file:{} error:{}'.format(function, filename, error))

	def config(self):
		self.session.open(MusicCenterSetup)

	def menuPressed(self):
		menulist=[(_("Configuration"), self.config)]
		#if not self.hlp.get('multiselectmode', False):
		#	menulist.append((_("Activate multiselectmode"), self.activateMultiselectmode),)
		#else:
		#	menulist.append((_("Dectivate multiselectmode"), self.deactivateMultiselectmode),)
		#self.selectedFiles = self["filelist"].getSelectedList()
		menulist.append((_("Sort by name (bouquet+)"), self.sortName),)
		menulist.append((_("Sort by date (bouquet-)"), self.sortDate),)
		if self["filelist"].canDescent():
			self.selectedDir=self["filelist"].getSelection()[0]
			menulist.append((_("(Re)Scan my musiclibrary"), self.addSelectedPathToDB),)
			if self.selectedDir in self.loadBookmarks():
				BMtext="Remove folder from Bookmarks"
				BMstring=self.delBookmark
			else:
				BMtext="Add folder to Bookmarks"
				BMstring=self.addBookmark
			if self["filelist"].getSelectionIndex()!=0:
				menulist.append((_(BMtext), BMstring),)
			menulist.append((_("Create new directory"), self.createNewDir),)
		else:
			filename=self["filelist"].getFilename()
			menulist.append((_("Create new directory"), self.createNewDir),)
			menulist.append((_("Create softlink..."),self.createLink),)

		self.session.openWithCallback(self.menuCallback, ChoiceBox, title=_("Options:\n"), list=menulist, keys=MYKEYSORT, titlebartext=_("MusicCenter"))

	def menuCallback(self, ret):
		ret and ret[1]()

	def activateMultiselectmode(self):
		self.hlp['multiselectmode']=True
		self["key_red"]=setText("Multi Loeschen")
		self["key_green"]=setText("Multi Umbenennen")
		self["key_yellow"]=setText("Multi Verschiebe/Kopiere")

	def deactivateMultiselectmode(self):
		self.hlp['multiselectmode']=False
		self["key_red"]=setText("Loesche")
		self["key_green"]=setText("Umbenennen")
		self["key_yellow"]=setText("Verschiebe/Kopiere")

	def execDeleteMultiFiles(self):
		self.session.open(MessageBox, _("Delete selected Files\n%s")%self.selectedFiles, type=MessageBox.TYPE_INFO, title="MusicCenter",timeout=10 )

	def addSelectedPathToDB(self):
		self.session.open(MusicCenterAddToDatabase, config.plugins.musiccenter.defaultfilebrowserpath.value)

	def addBookmark(self):
		bookmarks=self.loadBookmarks()
		bookmarks.append(self.selectedDir)
		with open(resolveFilename(SCOPE_CONFIG, 'mc_Bookmarks'), 'w') as lst:
			for l in bookmarks:
				lst.write(l+'\n')

	def delBookmark(self):
		bookmarks=self.loadBookmarks()
		for index, entry in enumerate(bookmarks):
			if(self.selectedDir == entry):
				logger.info('FileList]delBookmark]dir%s entry%s' %(self.selectedDir, entry))
				bookmarks.pop(index)
		with open(resolveFilename(SCOPE_CONFIG, 'mc_Bookmarks'), 'w') as lst:
			for l in bookmarks:
				lst.write(l+'\n')

	def createNewDir(self):
		self.session.openWithCallback(self.callbackNewDir, vInputBox, title=_(self["filelist"].getCurrentDirectory()), windowTitle=_("MusicCenter Create new folder in..."), text="name")

	def openHelp(self):
		help=''' MENU -> Show SubSelections
				AUDIO -> Back to Player if is running
				  PV  ->
				  RED -> Delete selected File/Folder from List, HDD and Database
				GREEN -> Move File/Folder
				YELLOW-> Copy File/Folder
				 BLUE -> Show Favorite Folders
				  0   -> One Folderlevel back
			'''

		self.session.open(Helps, 'Filebrowser', help)

	def createLink(self):
		self.session.openWithCallback(self.callbackCPmaniger, SoftLinkScreen, self["filelist"].getCurrentDirectory())

	def getListIndex(self, folder, filename):
		for index, file in enumerate(self["filelist"].getFileList()):
			logger.info('FileList]getListIndex]filename:{} file:{}'.format(filename, file[0][0]))
			if filename == file[0][0]:
				logger.info('FileList]getListIndex]match, folder:{} filename:{} foundindex:{}'.format(folder, filename, index))
				return index
		logger.debug('FileList]getListIndex]folder:{} filename:{} not found, return index:0'.format(folder, filename))
		return 0
#Ok
	def okPressed(self):
		if self["filelist"].canDescent(): # change directory
			self["filelist"].descent()
			self.updateLocationInfo()
		else:
			self.updateLastUsedDirectory()
			if not self.hlp.get('multiselectmode', False):
				self.songlisttmp=[]
				playableSongs=[]
				foundIndex=-1
				currfolder=self["filelist"].getCurrentDirectory()[:-1]
				selectedfilename=self["filelist"].getFilename()
				index=0
				for filename in sorted(os_listdir(currfolder)):
					if filename.lower().rsplit('.',1)[-1] in (PLAYABLEFORMATS):
						playableSongs.append(os_path_join(currfolder,filename))
						if selectedfilename == filename:
							foundIndex=index	
						index+=1

				if (foundIndex>=0):
					self.foundIndex=foundIndex
					self.playable_songs_count=len(playableSongs)
					scanPath.Start(playableSongs)
					self["waittext"].show()
					logger.info('MusicCenterFileList]okPressed]foundIndex:{} playable_songs_count:{}'.format(foundIndex, self.playable_songs_count))
					
				else:
					self.session.open(MessageBox, _("No music files found!"), type=MessageBox.TYPE_INFO,timeout=10 )
				playableSongs=[]
			else:
				self["filelist"].changeSelectionState()
				logger.info('MusicCenterFileList]okPressed]selectedFiles:{}'.format(self.selectedFiles))

	def gotThreadMsg(self, msg):
		state, count, songlist=scanPath.Message.pop()
		logger.debug('FileList]gotThreadMsg]state{} count{}'.format(state, count))
		if state in ('THREAD_WORKING', 'THREAD_SCAN_FINISHED'):
			self["waittext"].setText('Process {}/{}'.format(count, self.playable_songs_count))

			if state=='THREAD_SCAN_FINISHED':
				self.songlisttmp=songlist
				self.scanFinished()

		elif state=='THREAD_FINISHED': # database updated
			logger.debug('FileList]gotThreadMsg]THREAD_FINISHED database updated')
			self["waittext"].setText('')
			self["waittext"].hide()	

		elif state=='THREAD_EXCEPTED':
			t='Error read songlist->%s' %count
			logger.error('FileList]gotThreadMsg]THREAD_EXCEPTED set waittext->%s' %t)
			self["waittext"].setText(t)
		else:
			t='Read %s Error read file!' %(self.playable_songs_count)
			logger.error('FileList]gotThreadMsg]set waittext->%s' %t)
			self["waittext"].setText(t)

	def scanFinished(self):
		if not len(self.songlisttmp):
			logger.error('FileList]scanFinished]No songs found!')
			self.session.open(MessageBox, _("No songs found!"), type=MessageBox.TYPE_ERROR, timeout=100 )
			return
	
		if self.player is not None:
			if self.player.seekstate!=self.player.SEEK_STATE_STOP: # currentindex!=-2: #-2 is stopPlay
				logger.debug('Filelist]scanFinished]is player stopplay now...')
				self.player.stopPlay()
			logger.debug('Filelist]scanFinished]is player instance, delete now...')
			self.session.deleteDialog(self.player)
			self.player=None			
		try:
			if JCcheckModuleIsUpdated('Player'):
				reload(Player)
		except Exception, e:
			logger.exception('Filelist]scanFinished]exception')
			self.session.open(MessageBox, _("Error reload Player\n%s" %e), type=MessageBox.TYPE_ERROR, timeout=10 )
	
		logger.debug('Filelist]scanFinished]instantiate new player dialog')
		self.player = self.session.instantiateDialog(Player.Player, songlist=self.songlisttmp, index=self.foundIndex, playermode=FILEMODE, currentservice=self.currentservice, servicelist=self.servicelist)
		self.player.setShowHideAnimation("quick_fade")
		self.session.execDialog(self.player)
		self.songlisttmp=[]
		logger.debug('Filelist]scanFinished]remove waittext now')

	def directoryUp(self):
		self.refreshSongList()
		self.updateLocationInfo()

	def listBookmarks(self):
		bookmarklist=[(_("Cancel"), "BACK")]
		bookmarklist.append((_('MyMusic/'), config.plugins.musiccenter.defaultfilebrowserpath.value))
		bookmarks=self.loadBookmarks()
		for bookmark in bookmarks:
			if bookmark.startswith(config.plugins.musiccenter.defaultfilebrowserpath.value):
				showedbookmark='MyMusic/' + bookmark[len(config.plugins.musiccenter.defaultfilebrowserpath.value):]
			else:
				showedbookmark=bookmark
			bookmarklist.append((_(showedbookmark), bookmark))
		bookmarklist.sort()
		dei=self.session.openWithCallback(self.goToSelectedBookmark, ChoiceBox, title=_("My Bookmarks"), list=bookmarklist,keys=MYKEYSORT,  titlebartext=_("MusicCenter"))
		dei.setTitle(_("MusicCenter"))

	def goToSelectedBookmark(self, bookmark):
		if bookmark is not None:
			bookmark=bookmark[1]
			logger.info('Filelist]goToSelectedBookmark] use bookmark:{}'.format(bookmark))
			try:
				if bookmark.startswith("/"):
					self["filelist"].changeDir(bookmark)
					self.updateLocationInfo()
			except Exception:
				logger.exception('Filelist]goToSelectedBookmark]changeDir:{}'.format(bookmark))
		else:
			logger.info('Filelist]goToSelectedBookmark]no bookmark was selected{}'.format(bookmark))

	def loadBookmarks(self):
		bookmarks=[]
		if fileExists(resolveFilename(SCOPE_CONFIG, 'mc_Bookmarks')):
			with open(resolveFilename(SCOPE_CONFIG, 'mc_Bookmarks'), 'r') as bookmarklist:
				for bookmark in bookmarklist:
					bookmark=bookmark.strip()
					if os_path_exists(bookmark):
						bookmarks.append(bookmark)
					else:
						logger.error('MusicCenterFileList]init]bookmark:{} not found, skip'.format(bookmark))
		return bookmarks

	def execDelete(self):
		if not(self["filelist"].canDescent()):
			todelfn=self["filelist"].getFilename()
			self.hlp['todelfn']=todelfn
			dei=self.session.openWithCallback(self.callbackDeleteFile, MessageBox,_("Do you realy want to DELETE:\n" + todelfn), MessageBox.TYPE_YESNO)
			dei.setTitle(_("MusicCenter - Delete File"))
		elif (self["filelist"].getSelectionIndex()!=0) and (self["filelist"].canDescent()):
			to_delete_directory=self["filelist"].getSelection()[0]
			dei=self.session.openWithCallback(boundFunction(self.callbackDeleteDir, to_delete_directory), MessageBox,_("Do you realy want to DELETE:\n" + to_delete_directory + '\n\nYou do it at your own risk!'), MessageBox.TYPE_YESNO)
			dei.setTitle(_("MusicCenter - Delete Folder"))
		self.refreshSongList()

	def callbackDeleteFile(self, answer):
		if answer is True:
			logger.info('Filelist]callbackDeleteFile]start...')
			mypath=self["filelist"].getCurrentDirectory()
			filename=self.hlp.get('todelfn')
			todelfile=os_path_join(mypath, filename)
			deferToThread(JCdeleteSongfileonThread, todelfile, -1).addCallback(self.finishedcallbackDeleteFileJob,).addErrback(self.errorJob, 'callbackDeleteFile')

	def finishedcallbackDeleteFileJob(self, result):
		logger.info('Filelist]finishedcallbackDeleteFileJob]refresh SongList')
		self.refreshSongList()

	def refreshSongList(self):
		fl=self["filelist"]
		index=fl.getSelectionIndex()
		logger.info('Filelist]refreshSongList]refresh songlist and move to index:{}'.format(index))
		fl.changeDir(fl.getCurrentDirectory())
		fl.moveToIndex(index)

	def errorJob(self, error, jobname=''):
		logger.error('Filelist]errorJob]job->%s error->%s' %(jobname, error))

	def callbackDeleteDir(self, to_delete_directory, answer):
		logger.info('Filelist]callbackDeleteDir]to_delete_directory:{}'.format(to_delete_directory))
		if answer is True:
			mypath=self["filelist"].getCurrentDirectory()
			#fullfolder=self.hlp.get('todelfolder')
			logger.info('Filelist]callbackDeleteDir]delete folder')
			try:
				sh_rmtree(to_delete_directory)
			except Exception, e:
				self.session.open(MessageBox,_("{}\n-->{}".format(to_delete_directory, e)), type=MessageBox.TYPE_ERROR, timeout=6)
				logger.exception('Filelist]callbackDeleteDir]->%s')
			logger.info('Filelist]callbackDeleteDir]folder deleted')
			if to_delete_directory.startswith(config.plugins.musiccenter.defaultfilebrowserpath.value):# folder is in musiclib
				to_delete_directory=to_delete_directory[len(config.plugins.musiccenter.defaultfilebrowserpath.value):]
			logger.info('Filelist]callbackDeleteDir]db open delete folder:%s' %(to_delete_directory +'%'))
			deferToThread(self.deleteFolderOnThread, to_delete_directory +'%').addErrback(self.errorJob, 'callbackDeleteDir')
			logger.info('Filelist]callbackDeleteDir]done')
			self.refreshSongList()

	def deleteFolderOnThread(self, foldername):
		logger.info('Filelist]deleteFolderOnThread]start')
		#execSQLCommand('''DELETE FROM Songs WHERE filename LIKE ?;''', (foldername,))
		sql='''DELETE FROM Songs WHERE filename LIKE ?;'''
		args=(foldername,)
		res=execSQLCommandWithReturn(sql, args)
		logger.info('Filelist]deleteFolderOnThread]done')
			
	def ExecRename(self):
		logger.info('Filelist]ExecRename]start')
		if not(self["filelist"].canDescent()):
			RENfilename=self["filelist"].getFilename()
			self.session.openWithCallback(self.callbackExecRename, vInputBox, title=_("old:  "+RENfilename), windowTitle=_("MusicCenter Rename file..."), text=RENfilename)
		elif (self["filelist"].getSelectionIndex()!=0) and (self["filelist"].canDescent()):
			RENDIR=self["filelist"].getSelection()[0]
			self.session.openWithCallback(self.callbackRenDir, vInputBox, title=_("old:  "+RENDIR), windowTitle=_("MusicCenter Rename directory..."), text=RENDIR)

	def callbackExecRename(self, answer):
		logger.info('Filelist]callbackExecRename]answer->%s' %str(answer))
		if answer is not None:
			mypath=self["filelist"].getCurrentDirectory()
			source=mypath + self["filelist"].getFilename()
			dest=mypath + answer
			try:
				os_rename(source, dest)
			except OSError:
				dei=self.session.open(MessageBox,_("Rename: %s \nFAILED with OSError!" %answer), MessageBox.TYPE_ERROR)
				dei.setTitle(_("MusicCenter"))
				return
			try:
				pathoffset=len(config.plugins.musiccenter.defaultfilebrowserpath.value)
				logger.info('Filelist]callbackExecRename]songlist has len...')
				musiclibbasepath=config.plugins.musiccenter.defaultfilebrowserpath.value
				if source.startswith(musiclibbasepath):
					sourceindb=source[pathoffset:]
					destindb=dest[pathoffset:]
					sql='''UPDATE Songs SET filename=? WHERE filename=?;'''
					args=(destindb, sourceindb)
					execSQLCommand(sql, args)

			except Exception, e:
				logger.exception('Filelist]callbackExecRename]error:%s' %e)
				dei=self.session.open(MessageBox,_("Rename: %s \nFAILED! \n%s" % (answer,e)), MessageBox.TYPE_ERROR)
				dei.setTitle(_("MusicCenter"))

			finally:
				self.refreshSongList()

	def callbackRenDir(self, answer):
		logger.info('Filelist]callbackRenDir]answer->%s' %str(answer))
		if answer is not None:
			source=self["filelist"].getSelection()[0]
			dest=answer
			try:
				os_rename(source, dest)
				self.refreshSongList()


			except OSError:
				logger.exception('Filelist]callbackExecRename]OSError')
				dei=self.session.open(MessageBox,_("Rename: %s \nFAILED!\nOSError" %answer), MessageBox.TYPE_ERROR)
				dei.setTitle(_("MusicCenter"))
				return

			try:
				pathoffset=len(config.plugins.musiccenter.defaultfilebrowserpath.value)
				logger.info('Filelist]callbackExecRename]songlist has len...')
				musiclibbasepath=config.plugins.musiccenter.defaultfilebrowserpath.value
				if source.startswith(musiclibbasepath) and dest.startswith(musiclibbasepath):
					logger.info('Filelist]callbackExecRename]source and dest in musikdb update directorys')
					sourceindb=source[pathoffset:]
					destindb=dest[pathoffset:]
					sql='''UPDATE Songs SET filename = replace( filename, ?, ? ) WHERE filename LIKE ?;'''
					args=(sourceindb, destindb, sourceindb+"%")
					execSQLCommand(sql, args)
				elif source.startswith(musiclibbasepath) and not dest.startswith(musiclibbasepath):
					logger.info('Filelist]callbackExecRename]only source in musikdb delete directory from musikdb')
					sourceindb=source[pathoffset:]
					sql='''DELETE from Songs WHERE filename LIKE ?;'''
					args=(sourceindb+"%",)
					execSQLCommand(sql, args)
				elif not source.startswith(musiclibbasepath) and dest.startswith(musiclibbasepath):
					logger.info('Filelist]callbackExecRename]only dest in musikdb, scan directory to musikdb')
					PathToDatabase.Start(dest)
				else:
					logger.info('Filelist]callbackExecRename]only dest or nothing in musikdb do nothing')

			except Exception, e:
				logger.exception('Filelist]callbackExecRename]error')
				dei=self.session.open(MessageBox,_("Rename: %s \nFAILED! \n%s" %(answer,e)), MessageBox.TYPE_ERROR)
				dei.setTitle(_("MusicCenter"))

			finally:
				self.refreshSongList()

	def callbackNewDir(self, answer):
		if answer is None:
			return
		dest=self["filelist"].getCurrentDirectory()
		if (answer==""):
			dei=self.session.open(MessageBox,_("Directory name error !"), MessageBox.TYPE_ERROR)
			dei.setTitle(_("MusicCenter"))
		else:
			order=dest + answer
			try:
				if not pathExists(dest + answer):
					createDir(order)
			except:
				dei=self.session.open(MessageBox,_("%s \nFAILED!" % order), MessageBox.TYPE_ERROR)
				dei.setTitle(_("MusicCenter"))
			self.refreshSongList()

	def go2CPmaniger(self):
		fl=self["filelist"]
		mypath=fl.getCurrentDirectory()
		filename=fl.getFilename()
		idx=fl.getSelectionIndex()
		if not(fl.canDescent()):#file
			source=fl.getCurrentDirectory() + filename
		elif (idx!=0) and (fl.canDescent()): #folder
			source=fl.getSelection()[0]
		else:
			message=_('Filelist]go2CPmaniger]no valid location')
			logger.error(message)
			self.session.toastManager.showToast(message, 5)
			return
		logger.info('Filelist]go2CPmaniger]CopyMoveManageropen ')
		self.session.openWithCallback(self.callbackCPmaniger, CopyMoveManager, source)

	def callbackCPmaniger(self):
		logger.info('Filelist]callbackCPmaniger]moveToIndex list')
		self.refreshSongList()

	def callbackSetStartDir(self, answerSD):
		if answerSD is True:
			config.plugins.musiccenter.defaultfilebrowserpath.value=self["filelist"].getSelection()[0]
			config.plugins.musiccenter.defaultfilebrowserpath.save()
			configfile.save()

	def sortName(self):
		self["filelist"].sortName()
		try:
			message=_("[sort by Name] {}".format(self["filelist"].getCurrentDirectory()))
		except:
			message=_("MusicCenter Filelist")
		self.session.toastManager.showToast(message, 5)


	def sortDate(self):
		self["filelist"].sortDate()
		try:
			message=_("[sort by Date] {}".format(self["filelist"].getCurrentDirectory()))
		except:
			message=_("MusicCenter Filelist")
		self.session.toastManager.showToast(message, 5)


	def infoPressed(self):
		menuentrys=[]
		devices = self.readMobiles()	
		isdir = self["filelist"].canDescent()
		
		if not isdir:
			menuentrys.append((_("Add selected song to a Playlist"), self.startaddSongToPlaylist,),)
		menuentrys.append((_("Create new Playlist"), self.newPlaylist),)
	
		if not isdir:
			self.hlp['to_sync_file'] = os_path_join(self["filelist"].getSelection()[0], self["filelist"].getFilename())
			for dev in devices:
				menuentrys.append((_("Send selected song to {}".format(dev[0])), boundFunction(self.sendFileToMobile, dev)),)
				
		elif (self["filelist"].getSelectionIndex()!=0) and isdir:
			self.hlp['to_sync_directory']=self["filelist"].getSelection()[0]
			for dev in devices:
				menuentrys.append((_("Send selected Folder to {}".format(dev[0])), boundFunction(self.sendFolderToMobile, dev)),)

		self.session.openWithCallback(self.globalCallback, ChoiceBox, title=_("Please select device for sync!"), list=menuentrys,keys=MYKEYSORT, titlebartext=_("MusicCenter mobile device sync actions"))


	def startaddSongToPlaylist(self):
		song_id_length = self.hlp.get('song_id_length')
		if song_id_length is not None:
			song_id, length = song_id_length
			logger.debug('MusicCenterFileList]startaddSongToPlaylist]song_id:{}'.format(song_id))
			slentry = Item(songID = song_id, length = length)
			self.addSongToPlaylist(slentry)			
		else:
			self.session.open(MessageBox,_("something went wrong!"), MessageBox.TYPE_ERROR)


# to mobile folder
	def sendFolderToMobile(self, dev):
		logger.debug('MusicCenterFileList]sendFolderToMobile]dev:{}'.format(dev))
		to_sync_directory=self.hlp.get('to_sync_directory')
		self.hlp['dev']=dev
		dei=self.session.openWithCallback(self.sendFolderToMobileCallback, MessageBox,_("Do you realy want send folder:\n" + to_sync_directory + '\n\nYou do it at your own risk!'), MessageBox.TYPE_YESNO)
		dei.setTitle(_("MusicCenter - Sync Console"))
		
	def sendFolderToMobileCallback(self, ret):
		if ret is True: # rsync -e "ssh -p 22222"  -avr --progress --delete-after --modify-window=8 --human-readable --progress /hoeck/Local-320gb/Musik/2018-2 192.168.80.222:/storage/emulated/0/Music/
			logger.debug('MusicCenterFileList]sendFolderToMobileCallback]ret:{}'.format(ret))
			
			if not fileExists('/usr/bin/rsync'):
				self.session.open(MessageBox,_("rsync binary not found, please install!\nAbort!"), MessageBox.TYPE_ERROR)
				return
				
			if not fileExists('/usr/bin/sshpass'):
				self.session.open(MessageBox,_("sshpass binary not found, please install!\nAbort!"), MessageBox.TYPE_ERROR)
				return			
			#from subprocess import Popen as sp_Popen
			#p = sp_Popen('''echo "dummy" | nc 192.168.80.223 22222''', shell=True)

			#echo "dummy" | nc 192.168.80.223 22222

			to_sync_directory=self.hlp.pop('to_sync_directory')
			dev=self.hlp.pop('dev')
			musiclibbasepath=config.plugins.musiccenter.defaultfilebrowserpath.value
			pathoffset=len(musiclibbasepath)
			if to_sync_directory.startswith(musiclibbasepath): # path is in musiclib
				relative_directory=to_sync_directory[pathoffset:]
				logger.debug('MusicCenterFileList]sendFolderToMobileCallback]musiclibbasepath:{}'.format(musiclibbasepath))
				passwort, srcfolder, ip, port, dstfolder=dev[1],to_sync_directory , dev[2], dev[3], os_path_join(dev[4], relative_directory)
				make_directorys_comand='''sshpass -p "{}" ssh -p {} {} "mkdir -p {}"'''.format(passwort, port, ip, dstfolder)
				rsync_comand='''sshpass -p "{}" rsync -e "ssh -p {}" -azsv --no-perms --no-times --size-only --delete-after --stats --human-readable --progress  --exclude 'unsynced' "{}" {}:"{}"'''.format(passwort, port, srcfolder, ip, dstfolder)
				logger.debug('MusicCenterFileList]sendFolderToMobileCallback]befehl:{}'.format(make_directorys_comand))
				logger.debug('MusicCenterFileList]sendFolderToMobileCallback]befehl:{}'.format(rsync_comand))
				self.session.openWithCallback(self.rsyncFinishd, MusicCenterRsyncConsole, title='MusicCenter sync console', cmdlist = [make_directorys_comand, rsync_comand])
			else:
				self.session.open(MessageBox,_("To sync directory is not in musiclib, abort\n{}".format(to_sync_directory)), MessageBox.TYPE_ERROR)

	
	def rsyncFinishd(self):
		logger.debug('MusicCenterFileList]rsyncFinishd]...')
		self.session.open(MessageBox,_("Sync finished..."), MessageBox.TYPE_INFO)

		
# to mobile file	
	def sendFileToMobile(self, dev):
		logger.debug('MusicCenterFileList]sendFileToMobile]dev:{}'.format(dev))
		to_sync_file=self.hlp.get('to_sync_file')
		self.hlp['dev']=dev
		dei=self.session.openWithCallback(self.sendFileToMobileCallback, MessageBox,_("Do you realy want send file:\n" + to_sync_file + '\n\nYou do it at your own risk!'), MessageBox.TYPE_YESNO)
		dei.setTitle(_("MusicCenter - Sync Console"))


	def sendFileToMobileCallback(self, ret):
		if ret is True:
			logger.debug('MusicCenterFileList]sendFileToMobileCallback]ret:{}'.format(ret))

			if not fileExists('/usr/bin/rsync'):
				self.session.open(MessageBox,_("rsync binary not found, please install!\nAbort!"), MessageBox.TYPE_ERROR)
				return
				
			if not fileExists('/usr/bin/sshpass'):
				self.session.open(MessageBox,_("sshpass binary not found, please install!\nAbort!"), MessageBox.TYPE_ERROR)
				return			

			to_sync_file=self.hlp.pop('to_sync_file')
			dev=self.hlp.pop('dev')
			logger.debug('MusicCenterFileList]sendFileToMobileCallback]to_sync_file:{}'.format(to_sync_file))
			relative_directory='unsynced'
			passwort, srcfolder, ip, port, dstfolder=dev[1],to_sync_file , dev[2], dev[3], os_path_join(dev[4], relative_directory)
			make_directorys_comand='''sshpass -p "{}" ssh -p {} {} "mkdir -p {}"'''.format(passwort, port, ip, dstfolder)
			rsync_comand='''sshpass -p "{}" rsync -e "ssh -p {}" -azsv --no-perms --no-times --stats --human-readable --progress "{}" {}:"{}"'''.format(passwort, port, srcfolder, ip, dstfolder)
			logger.debug('MusicCenterFileList]sendFileToMobileCallback]befehl:{}'.format(make_directorys_comand))
			logger.debug('MusicCenterFileList]sendFileToMobileCallback]befehl:{}'.format(rsync_comand))
			self.session.openWithCallback(self.rsyncFinishd, MusicCenterRsyncConsole, title='MusicCenter sync console', cmdlist = [make_directorys_comand, rsync_comand]) # +' && '+rsync_comand]) # rsync_comand

	
	def readMobiles(self):
		devices_list=[]
		if fileExists('/etc/enigma2/mc_mobiledevices'):
			with open('/etc/enigma2/mc_mobiledevices', "rb") as f:
				for line in f:
					if not line.startswith('#'):
						devices_list.append(line.strip().split(','))
		return devices_list


	def missingSSHPass(self):
		comand = '''wget -O /tmp/sshpass_1.05-1_armhf.deb http://ftp.de.debian.org/debian/pool/main/s/sshpass/sshpass_1.05-1_armhf.deb'''	

		
	def explExit(self):
		logger.info('Filelist]explExit]Start close, hlp len is %d' %len(self.hlp))
		if scanPath.isRunning:
			logger.info('FileList]__onClose]scanPath remove')
			scanPath.Cancel()
		self.close(self.player) # isplayer)

	def videoPressed(self):
		options=[]
		self.session.openWithCallback(self.videoPressed_callback, ChoiceBox, title=_("Songlist Modify Actions\n"), list=options,keys=MYKEYSORT, titlebartext=_("MusicCenter"))

	def videoPressed_callback(self, ret):
		ret and ret[1]()

	def audioPressed(self):
		if self.player is not None and self.player.songlist:
			self.session.execDialog(self.player)

	def updateLastUsedDirectory(self):
		if config.plugins.musiccenter.rememberlastfilebrowserpath.value:
			tp=self["filelist"].getCurrentDirectory()
			logger.info('FileList]updateLastUsedDirectory]save lastusedpath:{}'.format(tp))
			config.plugins.musiccenter.lastusedpath.value=tp
			config.plugins.musiccenter.lastusedpath.save()
			configfile.save()
			
	def createSummary(self):
		if JCcheckModuleIsUpdated('LCDScreens'):
			reload(LCDScreens)
		return LCDScreens.LCDPlayer


	def __onClose(self):
		logger.info('FileList]__onClose]start, close...')
		self.updateLastUsedDirectory()
		self.scanPath_mc_conn=None
		#self.__dbpool.close()
		logger.info('FileList]__onClose]done.')


class CopyMoveManager(Screen):

	skin="""
		<screen position="center,center" size="1000,550" title="Select Copy/Move location...">
		<widget name="File" font="SansReg;20" halign="center" position="5,0" size="890,100" transparent="1" valign="center" zPosition="4"/>
		<widget name="CPto" position="5,100" scrollbarMode="showOnDemand" size="890,312" zPosition="4"/>
		<eLabel backgroundColor="#555555" position="5,420" size="890,2" zPosition="5"/>
		<ePixmap alphatest="on" pixmap="~/red.png" position="0,425" size="35,25" zPosition="5"/>
		<ePixmap alphatest="on" pixmap="~/yellow.png" position="310,425" size="35,25" zPosition="5"/>
		<eLabel font="SansReg;18" halign="left" position="35,425" size="120,25" text="MOVE" transparent="1" valign="center" zPosition="6"/>
		<eLabel font="SansReg;18" halign="left" position="345,425" size="120,25" text="COPY" transparent="1" valign="center" zPosition="6"/>
	</screen>"""

	def __init__(self, session, source="/tmp/none", idx=0):
	
		self.session=session
		self.skin_path='/usr/lib/enigma2/python/Plugins/Extensions/MusicCenter/images'
		
		Screen.__init__(self, session)
		
		self.src=source
		self["File"]=Label(_("WARNING! they doing now COPY or MOVE\n" + source + "\nto:"))
		self["CPto"]=MC_MultiFileSelectList([], config.plugins.musiccenter.copydestin.value, matchingPattern="^.*\.*")
		self["actions"]=ActionMap(["WizardActions", "ColorActions"],
		{
			"ok": self.okPressed,
			"back": self.NothingToDo,
			"red": self.startMoveJob,
			"yellow": self.startCopyJob
		}, -1)
		self.hlp={}
		self.scanPath_mc_conn=scanPath.MessagePump.recv_msg.connect(self.gotThreadMsg)

	def OneDescent(self):
		if self["CPto"].canDescent():
			self["CPto"].descent()

	def okPressed(self):
		if self["CPto"].canDescent():
			self["CPto"].descent()

	def NothingToDo(self):
		self.scanPath_mc_conn=None
		self.close()

	def startCopyJob(self):
		if self["CPto"].getSelectionIndex()>0:
			destfolder=self["CPto"].getSelection()[0]
			if config.plugins.musiccenter.copydestin.value!=destfolder:
				config.plugins.musiccenter.copydestin.value=destfolder
				config.plugins.musiccenter.copydestin.save()
				configfile.save()

			if os_path_isfile(self.src):
				self.copyFile(self.src, destfolder)

			elif os_path_isdir(self.src):
				dest_new_subfolder=self.src.strip('/').rsplit('/',1)[-1]
				destfolder=os_path_join(destfolder, dest_new_subfolder)
				t=SimpleThread(lambda:self.copyFolder(self.src, destfolder))
				t.deferred.addCallback(self.copyFolderCallback, destfolder)
				t.deferred.addErrback(self.errorcopyFolder,)
				t.start()
				dei=self.session.open(MessageBox,_("Copy:%s\nTo:%s\It takes a longer time" %(self.src, destfolder)), MessageBox.TYPE_INFO)
				dei.setTitle(_("MusicCenter"))

			self.scanPath_mc_conn=None
			self.close()

	def copyFile(self, srcfile, destfolder):
		logger.info('CopyMoveManager]copyFile]->%s ->%s' %(srcfile, destfolder))
		sh_copy2(srcfile, destfolder)
		srcfolder, srcfilename=os_path_split(srcfile)
		logger.info('CopyMoveManager]copyFile]scan file now for database:{}'.format(os_path_join(destfolder, srcfilename)))
		# add file to database
		to_scan_song=[(Item(text=srcfilename, filename=os_path_join(destfolder, srcfilename)),)]
		scanPath.Start(to_scan_song)

	def gotThreadMsg(self, msg):
		msg=scanPath.Message.pop()
		logger.info('CopyMoveManager]gotThreadMsg]:{}'.format(msg))
		if msg[0]=='THREAD_FINISHED': # database updated
			self.scanPath_mc_conn=None
			logger.info('CopyMoveManager]gotThreadMsg]')
		elif msg[0]=='THREAD_EXCEPTED':
			logger.error('CopyMoveManager]gotThreadMsg]THREAD_EXCEPTED:{}'.format(msg[1]))
			self.scanPath_mc_conn=None
		else:
			logger.error('CopyMoveManager]gotThreadMsg]was ist das? :{}'.format(msg))

	def copyFolder(self, srcfolder, destfolder):
		logger.info('CopyMoveManager]copyFolder]->%s ->%s' %(srcfolder, destfolder))
		sh_copytree(srcfolder, destfolder)
		logger.info('CopyMoveManager]copyFolder]scan folder new ->%s' %destfolder)
		pathToDatabase.Start(destfolder)

	def copyFolderCallback(self, result, destfolder):
		# dst is in db
		if destfolder.startswith(config.plugins.musiccenter.defaultfilebrowserpath.value):
			logger.info('CopyMoveManager]copyFolder]scan destfolder-->%s' %destfolder)
			pathToDatabase.Start(destfolder)

	def errorcopyFolder(self, error):
		logger.error('CopyMoveManager]errorcopyFolder]->%s' %error)

	def startMoveJob(self):
		if self["CPto"].getSelectionIndex()!=0:
			destfolder=self["CPto"].getSelection()[0]
			if config.plugins.musiccenter.copydestin.value!=destfolder:
				config.plugins.musiccenter.copydestin.value=destfolder
				config.plugins.musiccenter.copydestin.save()
				configfile.save()
			if os_path_isfile(self.src):
				self.moveFile(self.src, destfolder)
			elif os_path_isdir(self.src):
				self.startMoveFolder(self.src,destfolder)
			self.scanPath_mc_conn=None			
			self.close()

	def moveFile(self, srcfile, destfolder):
		logger.info('CopyMoveManager]moveFile] {} to{}'.format(srcfile, destfolder))
		sh_move(srcfile, destfolder)

		srcfolder, srcfilename=os_path_split(srcfile)
		if srcfile.startswith(config.plugins.musiccenter.defaultfilebrowserpath.value):# file is in musiclib
			srcfnindb=srcfile[len(config.plugins.musiccenter.defaultfilebrowserpath.value):]
		else:
			srcfnindb=srcfile
			
		if destfolder.startswith(config.plugins.musiccenter.defaultfilebrowserpath.value):# file is in musiclib
			dstfolderindb=destfolder[len(config.plugins.musiccenter.defaultfilebrowserpath.value):]
			dst_filenameindb=os_path_join(dstfolderindb+srcfilename)
			
			execSQLCommand('''UPDATE Songs SET filename=? WHERE filename=?;''', (dst_filenameindb, srcfnindb))
			
			dst_filename=os_path_join(destfolder+srcfilename)
			
			wavefn_src=buildWaveFilename(srcfile)
			if fileExists(wavefn_src):
				wavefn_dst=buildWaveFilename(dst_filename)
				sh_move(wavefn_src, wavefn_dst)
				
			coverfn_src=CoverLoader().getCoverfilename(srcfile)
			if fileExists(coverfn_src):
				coverfn_dst=CoverLoader().getCoverfilename(dst_filename)
				sh_move(coverfn_src, coverfn_dst)
			
		else:
			JCdeleteSongfileonThread(srcfile)

	def startMoveFolder(self, srcfolder, destfolder):
		self.hlp['move_srcfolder']=srcfolder
		self.hlp['move_destfolder']=destfolder
		self.session.openWithCallback(self.moveFolder, MessageBox, _("Start move\n{} to\n{}?\drink a coffee and lets move...".format(srcfolder, destfolder)), MessageBox.TYPE_YESNO)
	
	def moveFolder(self, yes):
		if yes:
			srcfolder=self.hlp.pop('move_srcfolder')
			destfolder=self.hlp.pop('move_destfolder')
			logger.info('CopyMoveManager]moveFolder]start sourse:{} dest:{}'.format(srcfolder, destfolder))
			try:
				sh_move(srcfolder, destfolder)
			except Exception, e:
				logger.exception('CopyMoveManager]moveFolder]...?')
				self.session.open(MessageBox,_("Move failed\n{}".format(e)), MessageBox.TYPE_ERROR)
				return
				
			logger.info('CopyMoveManager]moveFolder]done')
			# src is in db and dst is in db
			if srcfolder.startswith(config.plugins.musiccenter.defaultfilebrowserpath.value) and destfolder.startswith(config.plugins.musiccenter.defaultfilebrowserpath.value):
				logger.info('CopyMoveManager]moveFolder]src is in db and dst is in db, update entrys now...')
				srcfolderdb=srcfolder[len(config.plugins.musiccenter.defaultfilebrowserpath.value):]
				destfolderdb=destfolder[len(config.plugins.musiccenter.defaultfilebrowserpath.value):]
				sql='''UPDATE Songs SET filename = replace( filename, ?, ? ) WHERE filename LIKE ?;'''
				args=(srcfolderdb, destfolderdb, srcfolderdb +'%')
				execSQLCommand(sql,args)
			# src is in db and dst is not in db
			elif srcfolder.startswith(config.plugins.musiccenter.defaultfilebrowserpath.value) and not destfolder.startswith(config.plugins.musiccenter.defaultfilebrowserpath.value):
				logger.info('CopyMoveManager]moveFolder]src is in db and dst is not in db, clean entry in db')
				srcfolderdb=srcfolder[len(config.plugins.musiccenter.defaultfilebrowserpath.value):]
				sql='''DELETE FROM Songs WHERE filename LIKE ?;'''
				args=(srcfolderdb +'%',)
				execSQLCommand(sql,args)

			# src is not in db and dst is in db
			elif not srcfolder.startswith(config.plugins.musiccenter.defaultfilebrowserpath.value) and destfolder.startswith(config.plugins.musiccenter.defaultfilebrowserpath.value):
				logger.info('CopyMoveManager]moveFolder]src is not in db and dst is in db, scan destfolder now:{}'.format(destfolder))
				pathToDatabase.Start(destfolder)
			logger.info('CopyMoveManager]moveFolder]db updates done...')


class SoftLinkScreen(Screen):

	skin="""
		<screen position="center,center" size="900,450" title="Make a softlink...">
		<widget name="File" font="SansReg;20" halign="center" position="5,0" size="890,100" transparent="1" valign="center" zPosition="4"/>
		<widget name="SLto" position="5,100" scrollbarMode="showOnDemand" size="890,312" zPosition="4"/>
		<eLabel backgroundColor="#555555" position="5,420" size="890,2" zPosition="5"/>
		<ePixmap alphatest="on" pixmap="~/red.png" position="0,425" size="35,25" zPosition="5"/>
		<ePixmap alphatest="on" pixmap="~/yellow.png" position="310,425" size="35,25" zPosition="5"/>
		<eLabel font="SansReg;18" halign="left" position="35,425" size="120,25" text="Set name" transparent="1" valign="center" zPosition="6"/>
		<eLabel font="SansReg;18" halign="left" position="345,425" size="220,25" text="Make a softlink" transparent="1" valign="center" zPosition="6"/>
	</screen>"""

	def __init__(self, session, source="/tmp/"):
		self.skin=SoftLinkScreen.skin
		Screen.__init__(self, session)
		self.session=session
		self.skin_path='/usr/lib/enigma2/python/Plugins/Extensions/MusicCenter/images'
		self.src=source
		self.newSLname=" "
		self["File"]=Label("Set first the Softlink name ...")
		self["SLto"]=FileList([], '/', matchingPattern=None)
		self["actions"]=ActionMap(["WizardActions", "ColorActions"],
		{
			"ok": self.okPressed,
			"back": self.NothingToDo,
			"red": self.GetSLname,
			"yellow": self.MakeSLnow
		}, -1)

	def GetSLname(self):
		self.session.openWithCallback(self.callbackSetLinkName, vInputBox, title=_("Write the new softlink name here:"), windowTitle=_("MusicCenter"), text="newname")

	def callbackSetLinkName(self, answer):
		if answer is None:
			return
		if (" " in answer) or (answer==""):
			dei=self.session.open(MessageBox,_("Softlink name error !"), MessageBox.TYPE_ERROR)
			dei.setTitle(_("MusicCenter"))
			return
		else:
			self.newSLname=self.src + answer
			self["File"].setText(_("WARNING! they make now a softlink from\n" + self.newSLname + "\nto:"))

	def okPressed(self):
		if self["SLto"].canDescent():
			self["SLto"].descent()

	def NothingToDo(self):
		self.close(" ")

	def MakeSLnow(self):
		if self.newSLname!=" ":
			if self["SLto"].getSelectionIndex()!=0:
				if self["SLto"].canDescent():
					os_symlink(self["SLto"].getSelection()[0], self.newSLname)
				else:
					os_symlink((self["SLto"].getCurrentDirectory() + self["SLto"].getFilename()), self.newSLname)
				self.close(" ")
		else:
			dei=self.session.open(MessageBox,_("Softlink name error !"), MessageBox.TYPE_ERROR)
			dei.setTitle(_("MusicCenter"))

'''
atime: dieser Zeitstempel spiegelt den letzten Zugriff (lesend und schreibend) auf die Datei wieder
mtime: wann wurde das letzte mal in die Datei geschrieben. Wird also gesetzt wenn die Datei erstellt wird und jedes mal neu wenn sich am Inhalt der Datei was aendert
ctime: etwas missverstaendlich ChangeTime (nicht CreateTime!). Dieser Zeitstempel aendert sich wenn der Inhalt der Datei geaendert wird (also genau wie bei mtime), aber auch wenn sich an den Metadaten der Datei (Zugriffsrechte, Besitzer, ..) aendernFilelist]errorSimpleThread]
/usr/bin/jpg2mvi_dream.sh /Breesen/Steffen-Home/MusicCenterCache/musiccenter_artistpics/A/Abba/7065.jpg
'''
