# -*- coding: UTF-8 -*-

#  Music Center E2
#
#  Coded by bobo71 (c) 2014
#  Support: www.dreambox-tools.info
#
#  All Files of this Software are licensed under the Creative Commons
#  Attribution-NonCommercial-ShareAlike 3.0 Unported
#  License if not stated otherwise in a Files Head. To view a copy of this license, visit
#  http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a letter to Creative
#  Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.

#  Additionally, this plugin may only be distributed and executed on hardware which
#  is licensed by Dream Multimedia GmbH.

#  This plugin is NOT free software. It is open source, you are allowed to
#  modify it (if you keep the license), but it may not be commercially
#  distributed other than under the conditions noted above.
#  This applies to the source code as a whole as well as to parts of it, unless
#  explicitely stated otherwise.

from base64 import b64decode as base64_b64decode, b64encode as base64_b64encode
from mutagen.asf import ASF
from mutagen.easyid3 import EasyID3
from mutagen.flac import FLAC, Picture
from mutagen.id3 import ID3, APIC, PCNT, TIT2, TPE1, TALB, TCON, TDRC, TRCK, POPM, TPE2, WXXX, ID3NoHeaderError, Frames as id3_Frames
from mutagen.mp3 import MP3
from mutagen.mp4 import MP4, MP4Cover

from mutagen.oggvorbis import OggVorbis
from os import stat as os_stat, access as os_access, W_OK as os_W_OK, remove as os_remove
from os.path import exists as os_path_exists, realpath as os_path_realpath, join as os_path_join, realpath as os_path_realpath, getsize as os_path_getsize, split as os_path_split
from os.path import getmtime as os_path_getmtime
from PIL import Image
from shutil import move as os_move, copy as sh_copy2
from time import time as time_time
from threading import Thread

try:
	from Components.config import config
	from SQLConnector import execSQLCommand
	from Globals import CoverLoader
	from myLogger import logger
	e2isrunning=True
except:
	print 'tagger]Running outside enigma!'
	import logging as logger
	logger.basicConfig(level=logger.DEBUG)
	e2isrunning=False

'''
import sys
sys.path.append('/usr/lib/enigma2/python/Plugins/Extensions/MusicCenter')
import tagger as t
'''


def myenCode(value):
	returnValue="n/a"
	if value is not None:
		try:
			returnValue=value.encode("utf-8", 'ignore')
		except UnicodeDecodeError:
			try:
				returnValue=value.encode("iso8859-1", 'ignore')
			except UnicodeDecodeError:
				try:
					returnValue=value.decode("cp1252").encode("utf-8")
				except UnicodeDecodeError:
					try:
						returnValue=value.decode("utf-16").encode("utf-8")#, "replace")
					except UnicodeDecodeError:
						pass
	return returnValue


def getID3raw(filename, will_write=False):

	id3=None
	supported_filetype=''

	if filename.lower().endswith(".mp3"):

		if not will_write:
			try:
				id3 = MP3(filename)
				supported_filetype="MP3"
			except:
				logger.exception('getID3raw]Error get mp3 {}'.format(filename))
		else:
			try:
				id3 = ID3(filename)
				supported_filetype="MP3"

			except ID3NoHeaderError:
				logger.exception('getID3raw]exception ID3NoHeaderError, Adding ID3 header now')
				id3 = ID3()
				supported_filetype="MP3"
				logger.exception('create ID3 header done')

	elif filename.lower().endswith(".flac"):
		try:
			id3=FLAC(filename)
			supported_filetype="FLAC"
		except:
			logger.exception('getID3raw]Error get flac...')

	elif filename.lower().endswith(".ogg"):
		try:
			id3=OggVorbis(filename)
			supported_filetype="OGG"
		except:
			logger.exception('getID3raw]Error get ogg...')

	elif filename.lower().endswith(".wma"):
		try:
			id3=ASF(filename)
			supported_filetype="WMA"
		except:
			logger.exception('getID3raw]Error get wma...')

	elif filename.lower().endswith(".wav"):
		supported_filetype="WAV"
		id3 = True

	elif filename.lower().endswith(".m4a"):
		try:
			id3=MP4(filename)
			supported_filetype="M4A"
		except:
			logger.exception('getID3raw]Error get m4a...')

	else:
		logger.debug('getID3]no valid audotype')

	return id3, supported_filetype


def getID3Tags(root, filename, testing=False):

	class objectview(object):
		'''
		craete object attrib from dict
		'''
		def __init__(self, d):
			self.__dict__ = d

	title="n/a"
	genre="n/a"
	artist="n/a"
	album="n/a"
	tracknr='n/a'
	track="n/a"
	date="n/a"
	length="0"
	bitrate="n/a"
	albumartist="n/a"
	playcount=0
	rating=281
	wave=0
	hasapicframes=0
	youtubeurl=''

	#starttime=time_time()
	id3, supported_filetype=getID3raw(os_path_join(root, filename))
	#timing=time_time()-starttime
	#if timing > 0.5:
	#	f=os_path_join(root, filename)
	#	file_size=(os_path_getsize(f))
	#	print('getID3Tags]slow!! filename:{} size:{} scantime:{}'.format(f, file_size, timing))

	if supported_filetype:
		# length
		try:
			length=repr(id3.info.length)# myenCode(str(datetime_timedelta(seconds=int(id3.info.length))))
		except:
			length="0"
		#bitrate
		try:
			bitrate=id3.info.bitrate
		except:
			logger.exception('getID3Tags]get bitrate...')
			bitrate="n/a"
		# youtubeurl
		youtubeurl=getYoutubeurl(id3, supported_filetype)
		# waveform
		if getWavevalues(id3, supported_filetype):
			wave=1


		if id3:
			if supported_filetype=="WAV":
				pass

			elif supported_filetype=="WMA":
				#print ('getID3Tags]is wma %s' %str(id3))
				title=myenCode(map(unicode, id3.get('Title', [filename]))[0])
				genre=myenCode(map(unicode, id3.get('WM/Genre', ['n/a']))[0])
				artist=myenCode(map(unicode, id3.get('Author', ['n/a']))[0])
				album=myenCode(map(unicode, id3.get('WM/AlbumTitle', ['n/a']))[0])
				tracknr=myenCode(map(unicode, id3.get('WM/AlbumTitle', ['n/a']))[0])
				track=myenCode(map(unicode, id3.get('WM/Track', ['n/a']))[0])
				date=myenCode(map(unicode, id3.get('WM/Year', ['n/a']))[0])
				albumartist=myenCode(map(unicode, id3.get('WM/AlbumArtist', ['n/a']))[0])
				framesraw=id3.get('WM/Picture','')
				if framesraw:
					hasapicframes=1

			elif supported_filetype=="MP3":
				# Titel
				try:
					title=myenCode(id3.get('TIT2', [filename])[0])
					if not len(title):
						title=filename
				except:
					logger.exception('getID3Tags]MP3]title...')
					title=filename

				# Genre
				try:
					genre=myenCode(id3.get('TCON', ['n/a'])[0])
					if not len(genre):
						genre='n/a'
				except:
					logger.exception('getID3Tags]MP3]genre...')
					genre='n/a'

				#Artist
				try:
					artist=myenCode(id3.get('TPE1', ['n/a'])[0])
					if not len(artist):
						artist='n/a'
				except:
					logger.exception('getID3Tags]MP3]artist...')
					artist='n/a'

				# Album
				try:
					album=myenCode(id3.get('TALB', ['n/a'])[0])
					if not len(album):
						album='n/a'
				except:
					logger.exception('getID3Tags]MP3]album...')
					album='n/a'

				# Titelnummer
				try:
					track=myenCode(id3.get('TRCK', ['-1'])[0])
					if not len(track):
						track='-1'
					tracknr=track.split("/")[0]
				except:
					logger.exception('getID3Tags]MP3]track...')
					track='-1'

				# Datum
				try:
					date=myenCode(id3.get('TDRC', [''])[0]).split('-')[0]# new 16.04.2016
					#date=myenCode(id3.get('TDRC', ['n/a'])[0])
					if not date:
						date='n/a'
				except:
					logger.exception('getID3Tags]MP3]date...')
					date='n/a'

				# Albumartist
				try:
					albumartist=myenCode(id3.get('TPE2', ['n/a'])[0])
					if not albumartist:
						albumartist='n/a'
				except:
					logger.exception('getID3Tags]MP3]albumartist...')
					albumartist='n/a'

				#metadata=ID3(os_path_join(root, filename))
				playcount=id3.get('PCNT', PCNT(count=0)).count
				# rating das geht wohl auch besser...
				#rating=281
				popm=[ id3[i] for i in id3.iterkeys() if 'POPM' in i]
				if len(popm):
					rating=popm[0].rating

				#apic
				if id3.has_key('APIC:3'):
					hasapicframes=1#id3.get('APIC:Front cover').data
				else:
					frameslist=[id3[i] for i in id3.keys()if 'APIC:' in i]
					if len(frameslist):
						hasapicframes=1 # frameslist[0].data

			elif supported_filetype=="M4A":
				# Titel
				try:
					title=myenCode(id3.get('\xa9nam', [filename])[0])
					if not len(title):
						title=filename
				except:
					logger.exception('getID3Tags]M4A]title...')
					title=filename

				# Genre
				try:
					genre=myenCode(id3.get('\xa9gen', ['n/a'])[0])
					if not len(genre):
						genre='n/a'
				except:
					logger.exception('getID3Tags]M4A]genre...')
					genre='n/a'

				#Artist
				try:
					artist=myenCode(id3.get('\xa9ART', ['n/a'])[0])
					if not len(artist):
						artist='n/a'
				except:
					logger.exception('getID3Tags]M4A]artist...')
					artist='n/a'

				# Album
				try:
					album=myenCode(id3.get('\xa9alb', ['n/a'])[0])
					if not len(album):
						album='n/a'
				except:
					logger.exception('getID3Tags]M4A]album...')
					album='n/a'

				# Titelnummer
				try:
					track=myenCode(str(id3.get('trkn', [('-1',),])[0][0])) #track, total = v
					if not len(track):
						track='-1'
					tracknr=track.split("/")[0]
				except:
					logger.exception('getID3Tags]M4A]track...')
					track='-1'

				# Datum
				try:
					date=myenCode(id3.get('\xa9day', [''])[0]).split('-')[0]# new 16.04.2016
					if not date:
						date='n/a'
				except:
					logger.exception('getID3Tags]M4A]date...')
					date='n/a'

				# Albumartist
				try:
					albumartist=myenCode(id3.get('aART', ['n/a'])[0])
					if not albumartist:
						albumartist='n/a'
				except:
					logger.exception('getID3Tags]M4A]albumartist...')
					albumartist='n/a'

				#metadata=ID3(os_path_join(root, filename))
				playcount=id3.get('PCNT', PCNT(count=0)).count

				#rating
				rating=int('{}'.format(id3.get('----:enigma2.musiccenter:rating', ['281'])[0]))

				# waveform list
				#if ('----:enigma2.musiccenter:wave') in id3 and len(id3['----:enigma2.musiccenter:wave'][0][0]):
				#if getWavevalues(id3, supported_filetype):
				#	wave=1

				#cover
				if ('covr') in id3 and len(id3['covr'][0]):
					hasapicframes=1 # frameslist[0].data

			else: # all other files
				title=myenCode(id3.get('title', [filename])[0])
				if not len(title):
					title='n/a'
				try:
					genre=myenCode(id3.get('genre', ['n/a'])[0])
					if not len(genre):
						genre='n/a'
				except:
					genre="n/a"

				artist=myenCode(id3.get('artist', ['n/a'])[0])
				if not len(artist):
					artist='n/a'

				album=myenCode(id3.get('album', ['n/a'])[0])
				if not len(album):
					album='n/a'

				try:
					tracknr=myenCode(id3.get('tracknumber', ['-1'])[0].split("/")[0])
				except:
					tracknr='-1'

				track=myenCode(id3.get('tracknumber', ['n/a'])[0])

				date=myenCode(id3.get('date', ['n/a'])[0])
				if not len(date):
					date='n/a'

				try:
					albumartist=myenCode(id3.get('albumartist', [id3.get('album artist', ['n/a'])])[0])
				except:
					albumartist='n/a'

				# wave
				#if id3.has_key('WXXX:wave'):
				#	wave=1#id3.get('WXXX:wave')[0]

				if supported_filetype=='FLAC':
					# rating not conform with standard
					if id3.has_key('popm'):
						rating=int(myenCode(id3['popm'][0]))
					# playcount not conform with standard
					if id3.has_key('pcnt'):
						playcount=int(myenCode(id3['pcnt'][0]))
					# cover
					pics=id3.pictures
					if pics:
						frames=str(pics[0].data)
						if len(frames) > 0:
							hasapicframes =1 # frames
				elif supported_filetype=='OGG':
					# rating not conform with standard
					if id3.has_key('popm'):
						rating=int(myenCode(id3['popm'][0]))
					# playcount not conform with standard
					if id3.has_key('pcnt'):
						playcount=int(myenCode(id3['pcnt'][0]))
					# cover
					meta=id3.get('METADATA_BLOCK_PICTURE')
					if meta and len(meta)==1:
						hasapicframes=1 # Picture(base64_b64decode(meta[0])).data

					elif meta:
						iscoverfront=False
						for cov in meta:
							covtype=Picture(base64_b64decode(cov)).type
							if covtype==3: # 3-Cover front
								hasapicframes=1 # Picture(base64_b64decode(cov)).data
								#if hasapicframes:
								#	status=hasapicframes
								#	iscoverfront=True
								#	break
						if not iscoverfront:
							hasapicframes=True # Picture(base64_b64decode(meta[0])).data

			id3=True
			
		elif id3 is not None:
			title=filename.strip().rsplit('.',1)[0]
			id3=True
	else:
		title=filename.strip().rsplit('.',1)[0]

	logger.debug('getID3Tags]youtubeurl:{}'.format(youtubeurl))
	#logger.debug('\nid3:{}\ntitle:{}\ngenre:{}\nartist:{}\nalbum:{}\ntracknr:{}\ntrack:{}\ndate:{}\nlength:{}\nbitrate:{}\nalbumartist:{}\nsupported_filetype:{}\nplaycount:{}\nrating:{}\nwave:{}\nhasapicframes:{}\nyoutubeurl:{}'.format(id3, title, genre, artist, album, tracknr, track, date, length, bitrate, albumartist, supported_filetype, playcount, rating, wave, hasapicframes, youtubeurl))

	return objectview({
		"audio":id3,
		"title":title,
		"genre":genre,
		"artist":artist,
		"album":album,
		"tracknr":tracknr,
		"track":track,
		"date":date,
		"length":length,
		"bitrate":bitrate,
		"albumartist":albumartist,
		"filetype":supported_filetype,
		"playcount":playcount,
		"rating":rating,
		"wave":wave,
		"youtubeurl":youtubeurl,
		"hasapicframes":hasapicframes})


def getYoutubeurl(id3, supported_filetype):# global
	if supported_filetype=='MP3':
		return id3.get('WXXX:youtube', WXXX(url='')).url
	elif supported_filetype in ('OGG', 'FLAC'):
		return id3.get('wxxx:youtube', [''])[0]
	elif supported_filetype in ('M4A'):
		return '{}'.format(id3.get('----:enigma2.musiccenter:youtube',[''])[0])
	else:
		return ''

def getWavevalues(id3, supported_filetype):# global
	if supported_filetype=='MP3':
		return eval(id3.get('WXXX:wave', WXXX(url=[])).url)
	elif supported_filetype in ('OGG', 'FLAC'):
		return eval(id3.get('WXXX:wave', [str([])])[0])
	elif supported_filetype in ('M4A'):
		return eval(id3.get('----:enigma2.musiccenter:wave', [str([])])[0])
	else:
		return []


def bin_to_pic(image):
	'''
	convert bin to apicframes
	found on puddletag
	'''
	data = image.value
	(type, size) = struct_unpack_from("<bi", data)
	pos = 5
	mime = ""
	while data[pos:pos+2] != "\x00\x00":
		mime += data[pos:pos+2]
		pos += 2
	pos += 2
	description = ""
	while data[pos:pos+2] != "\x00\x00":
		description += data[pos:pos+2]
		pos += 2
	pos += 2
	image_data = data[pos:pos+size]
	return image_data


class ID3WriterThread(Thread):
	'''
	handle id3actions
	'''
	def __init__(self):

		Thread.__init__(self)

		self.__running=False
		self.__slentry=None
		self.__id3dict={}

	def __getRunning(self):
		return self.__running

	def __getProgress(self):
		return 0

	isRunning=property(__getRunning)
	theProgress=property(__getProgress)

	def Start(self, slentry, id3dict):

		if not self.__running:
			logger.debug('ID3WriterThread]Start]new thread now on {}'.format(slentry.filename))
			self.__running=True
			self.__slentry=slentry
			self.__id3dict=id3dict


			self.start()
		else:
			logger.error('ID3WriterThread]Start]old thread running abort!')


	def run(self):
		
		try:
			slentry=self.__slentry
			id3dict=self.__id3dict
			coverupdated=False
			changed=False

			if os_path_exists(slentry.filename) and os_access(slentry.filename, os_W_OK):

				id3, supported_filetype=getID3raw(slentry.filename, will_write=True)
				#logger.debug('ID3WriterThread]run]got id3 for {} is {} id3dict {}'.format(slentry.filename, supported_filetype, id3dict.keys()))

				if id3dict.has_key('cover'):
					#logger.info('ID3WriterThread]run]write cover')
					id3=self.embedCoverOnFile(slentry.filename, id3dict['cover'], id3, supported_filetype)
					coverupdated=True
					changed=True

				if id3dict.has_key('wavevalues'):
					#logger.info('ID3WriterThread]run]write wavevalues')

					wavevalues=id3dict['wavevalues']
					#logger.info('ID3WriterThread]run]WXXX:{}'.format(repr(wavevalues[:5])))
					if supported_filetype=='MP3':
						if id3.has_key(u'WXXX:wave'):
							wavevaluesold = id3.pop(u'WXXX:wave', 'n/a')
							logger.debug('ID3WriterThread]run]pop old wavevalues...')			
						id3['WXXX']=WXXX(encoding=3, desc=u'wave', url=u'%s' %repr(wavevalues))
						changed=True
					elif supported_filetype in ('FLAC','OGG'):
						id3['WXXX:wave']=u'%s' %repr(wavevalues)
						changed=True
					elif supported_filetype in('M4A'):
						id3['----:enigma2.musiccenter:wave']='%s' %repr(wavevalues)
						changed=True

				if id3dict.has_key('playcount'):
					#logger.info('ID3WriterThread]run]write playcount')
					if supported_filetype=='MP3':
						#logger.info('ID3WriterThread]run]increase playcount MP3')
						id3['PCNT']=PCNT(encoding=3, count=id3dict['playcount'])
						changed=True
					elif supported_filetype in ('FLAC','OGG'):
						#logger.info('ID3WriterThread]run]increase playcount FLAC OGG')
						id3['PCNT']='{}'.format(id3dict['playcount'])
						changed=True
					elif supported_filetype in('M4A'):
						id3['----:enigma2.musiccenter:playcount']='{}'.format(id3dict['playcount'])
						changed=True

				if id3dict.has_key('rating'):
					#logger.info('ID3WriterThread]run]write rating')
					rating=id3dict['rating']

					if rating in range(256):
						if supported_filetype=='MP3':
							oldrating=[ id3[i] for i in id3.iterkeys() if 'POPM' in i]
							if len(oldrating):
								#logger.info('ID3WriterThread]run]update rating:{} on POPM:{}'.format(rating,'POPM:'+oldrating[0].email))
								id3['POPM:'+oldrating[0].email].rating=rating
							else:
								#logger.info('ID3WriterThread]run]No Rating on Tag, add:{}'.format(rating))
								if supported_filetype=='MP3':
									#logger.info('ID3WriterThread]run]change rating MP3')
									id3['POPM']=POPM(email=u'no@email', rating=rating, count=0)# Mediamonkey
							changed=True
						elif supported_filetype in ('FLAC','OGG'):
							id3['POPM']='%s' %rating
							changed=True
						elif supported_filetype in('M4A'):
							id3['----:enigma2.musiccenter:rating']='{}'.format(rating)
							changed=True

					else:
						if supported_filetype=='MP3':
							oldrating=[ id3[id3key] for id3key in id3.iterkeys() if 'POPM' in id3key]
							for r in oldrating:
								#logger.info('ID3WriterThread]run]remove from ID3 Rating POPM {}'.format(r.email))
								id3.pop('POPM:{}'.format(r.email))
							changed=True
						elif supported_filetype in ('FLAC','OGG'):
							id3.__delitem__('POPM')
							changed=True
						elif supported_filetype in('M4A'):
							id3.__delitem__('----:enigma2.musiccenter:rating')
							changed=True

				if id3dict.has_key('artist'):
					#logger.info('ID3WriterThread]run]write artist')
					artist=id3dict['artist']
					if supported_filetype=="MP3":
						id3["TPE1"] = TPE1(encoding=3, text=unicode(artist, "utf-8"))
						changed=True
					elif supported_filetype in('OGG','FLAC'):
						id3["artist"] = unicode(artist, "utf-8")
						changed=True
					elif supported_filetype in('M4A'):
						id3['\xa9ART'] = unicode(artist, "utf-8")
						changed=True

				if id3dict.has_key('title'):
					#logger.info('ID3WriterThread]run]write title')
					if supported_filetype=="MP3":
						id3["TIT2"] = TIT2(encoding=3, text=unicode(id3dict['title'], "utf-8"))
						changed=True
					elif supported_filetype in('OGG','FLAC'):
						id3["title"] = unicode(id3dict['title'], "utf-8")
						changed=True
					elif supported_filetype in('M4A'):
						id3['\xa9nam'] = unicode(id3dict['title'], "utf-8")
						changed=True

				if id3dict.has_key('album'):
					#logger.info('ID3WriterThread]run]write album')
					if supported_filetype=="MP3":
						id3["TALB"] = TALB(encoding=3, text=unicode(id3dict['album'], "utf-8"))
						changed=True
					elif supported_filetype in('OGG','FLAC'):
						id3["album"] = unicode(id3dict['album'], "utf-8")
						changed=True
					elif supported_filetype in('M4A'):
						id3['\xa9alb'] = unicode(id3dict['album'], "utf-8")
						changed=True

				if id3dict.has_key('albumartist'):
					#logger.info('ID3WriterThread]run]write albumartist')
					if supported_filetype=="MP3":
						id3["TPE2"] = TPE2(encoding=3, text=unicode(id3dict['albumartist'], "utf-8"))
						changed=True
					elif supported_filetype in('OGG','FLAC'):
						id3["albumartist"] = unicode(id3dict['albumartist'], "utf-8")
						changed=True
					elif supported_filetype in('M4A'):
						id3['aART'] = unicode(id3dict['albumartist'], "utf-8")
						changed=True

				if id3dict.has_key('genre'):
					#logger.info('ID3WriterThread]run]write genre')
					if supported_filetype=="MP3":
						id3["TCON"] = TCON(encoding=3, text=unicode(id3dict['genre'], "utf-8"))
						changed=True
					elif supported_filetype in('OGG','FLAC'):
						id3["genre"] = unicode(id3dict['genre'], "utf-8")
						changed=True
					elif supported_filetype in('M4A'):
						id3['\xa9gen'] = unicode(id3dict['genre'], "utf-8")
						changed=True

				if id3dict.has_key('date'):
					#logger.info('ID3WriterThread]run]write date')
					if supported_filetype=="MP3":
						id3["TDRC"] = TDRC(encoding=3, text=unicode(id3dict['date'], "utf-8"))
						changed=True
					elif supported_filetype in('OGG','FLAC'):
						id3["date"] = unicode(id3dict['date'], "utf-8")
						changed=True
					elif supported_filetype in('M4A'):
						id3['\xa9day'] = unicode(id3dict['date'], "utf-8")
						changed=True

				if id3dict.has_key('tracknr'):
					#logger.info('ID3WriterThread]run]write tracknr')
					if supported_filetype=="MP3":
						id3["TRCK"] = TRCK(encoding=3, text=unicode(id3dict['tracknr'], "utf-8"))
						changed=True
					elif supported_filetype in('OGG','FLAC'):
						id3["tracknumber"] = unicode(id3dict['tracknr'], "utf-8")
						changed=True
					elif supported_filetype in('M4A'):
						id3['trkn'] = ((int(id3dict['tracknr']), 0),)
						changed=True

				if id3dict.has_key('youtubeurl'):
					youtubeurl = id3dict['youtubeurl']
					logger.debug('ID3WriterThread]run]write youtubeurl:{}'.format(youtubeurl))
					if supported_filetype=='MP3':
						if id3.has_key(u'WXXX:youtube'):
							youtubeurlold = id3.pop(u'WXXX:youtube', 'n/a')
							logger.debug('ID3WriterThread]run]pop old youtubeurl:{}'.format(youtubeurlold))
						id3['WXXX'] = WXXX(encoding=3,  desc=u'youtube', url=unicode(youtubeurl, "utf-8"))
						changed=True
					elif supported_filetype in ('FLAC','OGG'):
						id3["WXXX:youtube"] = unicode(youtubeurl, "utf-8")
						changed=True
					#The freeform ‘—-‘ frames use a key in the format ‘—-:mean:name’ where ‘mean’ is usually ‘com.apple.iTunes’ and ‘name’ is a unique identifier for this #frame. The value is a str, but is probably text that can be decoded as UTF-8. Multiple values per key are supported.
					elif supported_filetype in('M4A'):
						id3['----:enigma2.musiccenter:youtube'] ='{}'.format(youtubeurl)#unicode(youtubeurl, "utf-8")
						changed=True

				if changed:# save now...

					logger.info('ID3WriterThread]run]id3 changed, start write')
					id3.save(slentry.filename)
					logger.info('ID3WriterThread]run]id3 write done')

					if slentry.filename.startswith(config.plugins.musiccenter.defaultfilebrowserpath.value) and e2isrunning:# file is in my musiclib
						self.updateSongDBonThread(slentry)
					else:
						logger.info('ID3WriterThread]run]file not in db, skip')

					if coverupdated: # for timing here...
						CoverLoader().load(slentry.filename)
				else:
					logger.error('ID3WriterThread]run]nothing changed???')
			else:
				logger.error('ID3WriterThread]run]file is missing, or no writeaccess, abort!!!!')			
		except:
			logger.exception('ID3WriterThread]run]error!!!!')
			
		self.__jobtype=''
		self.__running=False
		self.__slentry=None
		self.__id3dict={}

		Thread.__init__(self)


	def updateSongDBonThread(self, slentry):
		song_id=slentry.songID
		root, filename=os_path_split(slentry.filename)
		ID3=getID3Tags(root, filename)
		file_m_time=(os_path_getmtime(slentry.filename))
		file_size=(os_path_getsize(slentry.filename))
		fullfnindb=slentry.filename[len(config.plugins.musiccenter.defaultfilebrowserpath.value):]
		sql='''UPDATE Songs SET filename=?, title=?, artist=?, album=?, albumartist=?, genre=?, date=?, tracknumber=?, bitrate=?, length=?, track=?, file_m_time=?, file_size=?, playcount=?, rating=?, wave=?, cover=?, youtubeurl=? WHERE song_id=?;'''
		args=(fullfnindb, ID3.title, ID3.artist, ID3.album, ID3.albumartist, ID3.genre, ID3.date, ID3.tracknr, ID3.bitrate, ID3.length, ID3.track, file_m_time, file_size, ID3.playcount, ID3.rating, ID3.wave, ID3.hasapicframes, ID3.youtubeurl, song_id)
		execSQLCommand(sql,args)

	def getimageColorDepth(self, imobject):
		mode_to_bpp = {'1':1, 'L':8, 'P':8, 'RGB':24, 'RGBA':32, 'CMYK':32, 'YCbCr':24, 'I':32, 'F':32}
		return mode_to_bpp.get(imobject.mode, 0)


	def embedCoverOnFile(self, filename, coverfile, id3, supported_filetype):

		logger.info('TagWriter]embedCoverOnFile]song:{} coverfn:{}'.format(filename, coverfile))

		if supported_filetype=='MP3':
			# remove all pics
			for tagentry in id3.iteritems():
				if 'APIC' in tagentry[0] and id3[tagentry[0]].type == 3:
					id3.pop(tagentry[0])
			imagedata=open(coverfile, 'rb').read()
			id3['APIC']=APIC(3, 'image/jpg', 3, 'Cover (front)', imagedata)

		elif supported_filetype=='FLAC':
			im=Image.open(coverfile)
			img=Picture()

			pics = id3.pictures
			id3.clear_pictures()

			for pic in pics:
				if pic.type != 3:
					id3.add_picture(pic)

			with open(coverfile,'rb') as coverfnfile:
				imgdata=coverfnfile.read()

			img.mime=u'image/jpeg'
			img.type=3
			img.desc=u'Cover (front)'
			img.data=imgdata
			img.width=im.size[0]
			img.height=im.size[1]
			img.depth=self.getimageColorDepth(im)
			id3.add_picture(img)

		elif supported_filetype=='OGG':

			new_METADATA_BLOCK_PICTURE=[]
			if id3.has_key('METADATA_BLOCK_PICTURE'):
				logger.debug('TagWriter]embedCoverOnFile]oggfile has covers, check for old Frontcover...')
				for cov in iter(id3['METADATA_BLOCK_PICTURE']):
					covtype=Picture(base64_b64decode(cov)).type
					if covtype==3:
						logger.debug('TagWriter]embedCoverOnFile]has old Frontcover, remove..')
					else:
						new_METADATA_BLOCK_PICTURE.append(cov)

			im=Image.open(coverfile)
			img=Picture()

			with open(coverfile,'rb') as cf:
				data=cf.read()

			img.data=data
			img.type=3
			img.desc=u'Cover (front)'
			img.mime=u'image/jpeg'
			img.width=im.size[0]
			img.height=im.size[1]
			img.depth=self.getimageColorDepth(im)

			tmp=img.write()
			tmp=base64_b64encode(tmp).decode('ascii') # new front cover

			new_METADATA_BLOCK_PICTURE.append(tmp)

			id3['METADATA_BLOCK_PICTURE']=new_METADATA_BLOCK_PICTURE

		elif supported_filetype=='M4A':
			imageformat=Image.open(coverfile).format
			if imageformat=='PNG':
				imageformat=MP4Cover.FORMAT_PNG
			else:
				imageformat=MP4Cover.FORMAT_JPEG

			with open(coverfile,'rb') as cf:
				id3["covr"] = [ MP4Cover(cf.read(), imageformat=imageformat) ]

		else:
			logger.error('TagWriter]EmbedCoverToFile]Filetype not suppoerted for coverembed! %s' %filename)

		return id3


id3writerthreadinstance=ID3WriterThread()
